from celescope.__init__ import __CONDA__
from celescope.vdj.__init__ import __STEPS__, __ASSAY__
from celescope.tools.Multi import Multi


class Multi_vdj(Multi):
    def custome_args(self):
        self.parser.add_argument("--type", help='TCR or BCR', required=True)
        self.parser.add_argument(
        '--iUMI', help='minimum number of UMI of identical receptor type and CDR3', default=1)
        self.parser.add_argument('--thread', help='thread', default=6)

    def read_custome_args(self):
        self.iUMI =  self.args.iUMI
        self.type = self.args.type 
        self.thread = self.args.thread

    def mapping_vdj(self, sample):
        step = 'mapping_vdj'
        fq = f'{self.outdir_dic[sample]["cutadapt"]}/{sample}_clean_2.fq.gz'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--fq {fq} '
            f'--type {self.type} '
            f'--thread {self.thread} '
        )
        self.generate_other(cmd, step, sample, m=15, x=self.thread)


    def count_vdj(self, sample):
        # count_vdj
        step = 'count_vdj'
        UMI_count_filter1_file = (
            f'{self.outdir_dic[sample]["mapping_vdj"]}/{sample}_UMI_count_filtered1.tsv'
        )
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--type {self.type} '
            f'--iUMI {self.iUMI} '
            f'--UMI_count_filter1_file {UMI_count_filter1_file} '
            f'--match_dir {self.col4_dict[sample]} '
        )
        self.generate_other(cmd, step, sample, m=8, x=self.thread)


def main():
    multi = Multi_vdj(__ASSAY__, __STEPS__, __CONDA__)
    multi.col4_default = None
    multi.run()

if __name__ == '__main__':
    main()




from collections import defaultdict
import os
import pysam
from .utils import log, read_one_col
from .utils import seq_ranges, get_scope_bc, parse_pattern


class Chemistry():

    def __init__(self, fq1):
        self.fq1 = fq1
        self.nRead = 10000
        self.T4 = 0
        self.L57C = 0

    @log
    def get_chemistry(self):
        '''
        'scopeV2.0.1': 'C8L16C8L16C8L1U8T18'
        'scopeV2.1.1': 'C8L16C8L16C8L1U12T18'
        'scopeV2.2.1': 'C8L16C8L16C8L1U12T18' with 4 types of linkers
        '''
        # init
        linker_4_file, _whitelist = get_scope_bc('scopeV2.2.1')
        linker_4_list, _num = read_one_col(linker_4_file)
        linker_4_dict = defaultdict(int)
        pattern_dict = parse_pattern('C8L16C8L16C8L1U12T18')

        with pysam.FastxFile(self.fq1) as fh:
            for i in range(self.nRead):
                entry = fh.__next__()
                seq = entry.sequence
                L57C = seq[56]
                if L57C == 'C':
                    self.L57C += 1
                T4 = seq[65:69]
                if T4 == 'TTTT':
                    self.T4 += 1
                linker = seq_ranges(seq, pattern_dict=pattern_dict['L'])
                if linker in linker_4_list:
                    linker_4_dict[linker] += 1

        self.percent_T4 =  self.T4 / self.nRead 
        self.percent_L57C = self.L57C / self.nRead
        Chemistry.get_chemistry.logger.info(f'percent T4: {self.percent_T4}')
        Chemistry.get_chemistry.logger.info(f'percent L57C: {self.percent_L57C}')
        if self.percent_T4 > 0.5:
            self.chemistry = 'scopeV2.0.1'
        else:
            # V2.1.1 or V2.2.1 or failed
            self.valid_linker_type = 0
            for linker in linker_4_dict:
                linker_4_dict[linker] = linker_4_dict[linker] / self.nRead
                if linker_4_dict[linker] > 0.05:
                    self.valid_linker_type += 1
            self.linker_4_dict = linker_4_dict
            Chemistry.get_chemistry.logger.info(self.linker_4_dict)
            if self.valid_linker_type == 0:
                raise Exception('auto chemistry detection failed!')
            elif self.valid_linker_type == 1:
                self.chemistry = 'scopeV2.1.1'
            elif self.valid_linker_type < 4:
                self.chemistry = 'scopeV2.1.1'
                Chemistry.get_chemistry.logger.warning(
                    f'chemistry scopeV2.2.1 only has {self.valid_linker_type} linker types!')
            else:
                self.chemistry = 'scopeV2.2.1'
        Chemistry.get_chemistry.logger.info(f'chemistry: {self.chemistry}')
        return self.chemistry






import unittest
import os
import pandas as pd
import glob
from celescope.snp.snpCalling import convert, call_all_snp, call_snp, split_bam, summary, read_index
from celescope.tools.utils import read_barcode_file, glob_genomeDir
from .analysis_snp import analysis_variant


class test_snp(unittest.TestCase):
    def setUp(self):
        os.chdir('/SGRNJ01/RD_dir/pipeline_test/zhouyiqi/unittest/snp')
        self.genomeDir = '/SGRNJ/Public/Database/genome/homo_sapiens/ensembl_92'
        self.gene_list_file = './gene_list.tsv'
        self.index_file = './S20070818_TS/05.snpCalling/S20070818_TS_cell_index.tsv'
        self.thread = 20
        self.outdir = './S20070818_TS/05.snpCalling/'
        self.analysis_outdir = './S20070818_TS/06.analysis_snp'
        _refFlat, self.gtf, self.fasta = glob_genomeDir(self.genomeDir, fa=True)
        self.match_dir = '/SGRNJ02/RandD4/RD20051303_Panel/20200717/S20070818_ZL/'
        self.sample = 'S20070818_TS'
        self.count_file = './S20070818_TS/05.snpCalling/S20070818_TS_count.tsv'
        self.vcf_file = './S20070818_TS/05.snpCalling/S20070818_TS_anno.vcf'
        self.assay = 'snp'
        self.annovar_config = '/SGRNJ01/RD_dir/pipeline_test/zhouyiqi/soft/annovar/annovar.config'
        self.step_analysis_variant = analysis_variant(
            self.analysis_outdir,
            self.sample,
            self.match_dir,
            self.vcf_file,
            self.index_file,
            self.assay,
            self.annovar_config,
        )

    @unittest.skip('pass')
    def test_convert(self):
        _refFlat, gtf = glob_genomeDir(self.genomeDir)
        gene_list = convert(self.gene_list_file, gtf)
        print(gene_list)

    @unittest.skip('pass')
    def test_call_all_snp(self):
        call_all_snp(self.index_file, self.outdir, self.thread, self.fasta)

    @unittest.skip('pass')
    def test_call_snp(self):
        index = 526
        call_snp(index, self.outdir, self.fasta)

    @unittest.skip('pass')
    def test_split_bam(self):
        bam = './S20070818_TS/04.featureCounts/S20070818_TS_name_sorted.bam'
        barcodes, _nCell = read_barcode_file(self.match_dir)
        gene_id_name_dic = convert(self.gene_list_file, self.gtf)
        min_query_length = 35
        split_bam(
            bam, barcodes, self.outdir,
            self.sample, gene_id_name_dic, min_query_length)
    
    @unittest.skip('pass')
    def test_summary(self):
        summary(self.index_file, self.count_file, self.outdir, self.sample)

    @unittest.skip('pass')
    def test_index(self):
        index_file = '/SGRNJ02/RandD4/RD20051303_Panel/20200929/S20070817_TS/05.snpCalling/S20070817_TS_cell_index.tsv'
        df_index, df_valid = read_index(index_file)
        index_arg = df_valid.index
        print(index_arg)
        print(list(index_arg))
    
    @unittest.skip('pass')
    def test_annovar(self):
        self.step_analysis_variant.annovar()
    
    def test_parse_annovar(self):
        from celescope.tools.utils import parse_annovar
        self.annovar_file = glob.glob(f'{self.analysis_outdir}/{self.sample}*_multianno.txt')[0]
        parse_annovar(self.annovar_file)



if __name__ == '__main__':
    unittest.main()



#!/bin/env python
# coding=utf8

import argparse
from celescope.__init__ import __VERSION__, ASSAY_DICT


def main():
    parser = argparse.ArgumentParser(description='CeleScope')
    parser.add_argument(
        '-v',
        '--version',
        action='version',
        version=__VERSION__)
    subparsers = parser.add_subparsers()

    # rna
    assay = 'rna'
    text = ASSAY_DICT[assay]
    subparsers_rna = subparsers.add_parser(assay, help=text, description=text)
    subparsers_rna_sub = subparsers_rna.add_subparsers()

    from celescope.tools.sample_info import sample_info, get_opts_sample
    parser_sample = subparsers_rna_sub.add_parser('sample')
    get_opts_sample(parser_sample, True)
    parser_sample.set_defaults(func=sample_info)

    from celescope.tools.barcode import barcode, get_opts_barcode
    parser_barcode = subparsers_rna_sub.add_parser('barcode')
    get_opts_barcode(parser_barcode, True)
    parser_barcode.set_defaults(func=barcode)

    from celescope.tools.cutadapt import cutadapt, get_opts_cutadapt
    parser_cutadapt = subparsers_rna_sub.add_parser('cutadapt')
    get_opts_cutadapt(parser_cutadapt, True)
    parser_cutadapt.set_defaults(func=cutadapt)

    from celescope.tools.STAR import STAR, get_opts_STAR
    parser_STAR = subparsers_rna_sub.add_parser('STAR')
    get_opts_STAR(parser_STAR, True)
    parser_STAR.set_defaults(func=STAR)

    from celescope.tools.featureCounts import featureCounts, get_opts_featureCounts
    parser_featureCounts = subparsers_rna_sub.add_parser('featureCounts')
    get_opts_featureCounts(parser_featureCounts, True)
    parser_featureCounts.set_defaults(func=featureCounts)

    from celescope.tools.count import count, get_opts_count
    parser_count = subparsers_rna_sub.add_parser('count')
    get_opts_count(parser_count, True)
    parser_count.set_defaults(func=count)

    from celescope.tools.analysis import analysis, get_opts_analysis
    parser_analysis = subparsers_rna_sub.add_parser('analysis')
    get_opts_analysis(parser_analysis, True)
    parser_analysis.set_defaults(func=analysis)

    from celescope.rna.run import run
    parser_run = subparsers_rna_sub.add_parser(
        'run', help='run all steps', conflict_handler='resolve')
    get_opts_sample(parser_run, False)
    get_opts_barcode(parser_run, False)
    get_opts_cutadapt(parser_run, False)
    get_opts_STAR(parser_run, False)
    get_opts_featureCounts(parser_run, False)
    get_opts_count(parser_run, False)
    get_opts_analysis(parser_run, False)
    parser_run.set_defaults(func=run)

    # rna_virus
    assay = 'rna_virus'
    text = ASSAY_DICT[assay]
    subparsers_rna_virus = subparsers.add_parser(
        assay, help=text, description=text)
    subparsers_rna_virus_sub = subparsers_rna_virus.add_subparsers()

    parser_sample = subparsers_rna_virus_sub.add_parser(
        'sample', description='sample infomation')
    get_opts_sample(parser_sample, True)
    parser_sample.set_defaults(func=sample_info)

    from celescope.tools.barcode import barcode, get_opts_barcode
    parser_barcode = subparsers_rna_virus_sub.add_parser('barcode')
    get_opts_barcode(parser_barcode, True)
    parser_barcode.set_defaults(func=barcode)

    from celescope.tools.cutadapt import cutadapt, get_opts_cutadapt
    parser_cutadapt = subparsers_rna_virus_sub.add_parser('cutadapt')
    get_opts_cutadapt(parser_cutadapt, True)
    parser_cutadapt.set_defaults(func=cutadapt)

    from celescope.tools.STAR import STAR, get_opts_STAR
    parser_STAR = subparsers_rna_virus_sub.add_parser('STAR')
    get_opts_STAR(parser_STAR, True)
    parser_STAR.set_defaults(func=STAR)

    from celescope.rna_virus.STAR_virus import STAR_virus, get_opts_STAR_virus
    parser_STAR_virus = subparsers_rna_virus_sub.add_parser('STAR_virus')
    get_opts_STAR_virus(parser_STAR_virus, True)
    parser_STAR_virus.set_defaults(func=STAR_virus)

    from celescope.tools.featureCounts import featureCounts, get_opts_featureCounts
    parser_featureCounts = subparsers_rna_virus_sub.add_parser('featureCounts')
    get_opts_featureCounts(parser_featureCounts, True)
    parser_featureCounts.set_defaults(func=featureCounts)

    from celescope.tools.count import count, get_opts_count
    parser_count = subparsers_rna_virus_sub.add_parser('count')
    get_opts_count(parser_count, True)
    parser_count.set_defaults(func=count)

    from celescope.rna_virus.count_virus import count_virus, get_opts_count_virus
    parser_count_virus = subparsers_rna_virus_sub.add_parser('count_virus')
    get_opts_count_virus(parser_count_virus, True)
    parser_count_virus.set_defaults(func=count_virus)

    from celescope.rna_virus.analysis_rna_virus import analysis_rna_virus, get_opts_analysis_rna_virus
    parser_analysis_rna_virus = subparsers_rna_virus_sub.add_parser(
        'analysis_rna_virus')
    get_opts_analysis_rna_virus(parser_analysis_rna_virus, True)
    parser_analysis_rna_virus.set_defaults(func=analysis_rna_virus)

    from celescope.rna_virus.run import run
    parser_run = subparsers_rna_virus_sub.add_parser(
        'run', help='run all steps', conflict_handler='resolve')
    get_opts_sample(parser_run, False)
    get_opts_barcode(parser_run, False)
    get_opts_cutadapt(parser_run, False)
    get_opts_STAR(parser_run, False)
    get_opts_STAR_virus(parser_run, False)
    get_opts_featureCounts(parser_run, False)
    get_opts_count_virus(parser_run, False)
    get_opts_analysis_rna_virus(parser_run, False)
    parser_run.set_defaults(func=run)

    # capture_virus
    assay = 'capture_virus'
    text = ASSAY_DICT[assay]
    subparsers_capture_virus = subparsers.add_parser(
        assay, help=text, description=text)
    subparsers_capture_virus_sub = subparsers_capture_virus.add_subparsers()

    parser_sample = subparsers_capture_virus_sub.add_parser('sample')
    get_opts_sample(parser_sample, True)
    parser_sample.set_defaults(func=sample_info)

    parser_barcode = subparsers_capture_virus_sub.add_parser('barcode')
    get_opts_barcode(parser_barcode, True)
    parser_barcode.set_defaults(func=barcode)

    parser_cutadapt = subparsers_capture_virus_sub.add_parser('cutadapt')
    get_opts_cutadapt(parser_cutadapt, True)
    parser_cutadapt.set_defaults(func=cutadapt)

    parser_STAR_virus = subparsers_capture_virus_sub.add_parser('STAR_virus')
    get_opts_STAR_virus(parser_STAR_virus, True)
    parser_STAR_virus.set_defaults(func=STAR_virus)

    from celescope.capture_virus.count_capture_virus import count_capture_virus, get_opts_count_capture_virus
    parser_count_capture_virus = subparsers_capture_virus_sub.add_parser(
        'count_capture_virus')
    get_opts_count_capture_virus(parser_count_capture_virus, True)
    parser_count_capture_virus.set_defaults(func=count_capture_virus)

    from celescope.capture_virus.analysis_capture_virus import analysis_capture_virus, get_opts_analysis_capture_virus
    parser_analysis_capture_virus = subparsers_capture_virus_sub.add_parser(
        'analysis_capture_virus')
    get_opts_analysis_capture_virus(parser_analysis_capture_virus, True)
    parser_analysis_capture_virus.set_defaults(func=analysis_capture_virus)

    from celescope.capture_virus.run import run
    parser_run = subparsers_capture_virus_sub.add_parser(
        'run', help='run all steps', conflict_handler='resolve')
    get_opts_sample(parser_run, False)
    get_opts_barcode(parser_run, False)
    get_opts_cutadapt(parser_run, False)
    get_opts_STAR_virus(parser_run, False)
    get_opts_count_capture_virus(parser_run, False)
    parser_run.set_defaults(func=run)

    # fusion
    assay = 'fusion'
    text = ASSAY_DICT[assay]
    subparsers_fusion = subparsers.add_parser(
        assay, help=text, description=text)
    subparsers_fusion_sub = subparsers_fusion.add_subparsers()

    parser_sample = subparsers_fusion_sub.add_parser('sample')
    get_opts_sample(parser_sample, True)
    parser_sample.set_defaults(func=sample_info)

    parser_barcode = subparsers_fusion_sub.add_parser('barcode')
    get_opts_barcode(parser_barcode, True)
    parser_barcode.set_defaults(func=barcode)

    parser_cutadapt = subparsers_fusion_sub.add_parser('cutadapt')
    get_opts_cutadapt(parser_cutadapt, True)
    parser_cutadapt.set_defaults(func=cutadapt)

    from celescope.fusion.STAR_fusion import STAR_fusion, get_opts_STAR_fusion
    parser_STAR_fusion = subparsers_fusion_sub.add_parser('STAR_fusion')
    get_opts_STAR_fusion(parser_STAR_fusion, True)
    parser_STAR_fusion.set_defaults(func=STAR_fusion)

    from celescope.fusion.count_fusion import count_fusion, get_opts_count_fusion
    parser_count_fusion = subparsers_fusion_sub.add_parser('count_fusion')
    get_opts_count_fusion(parser_count_fusion, True)
    parser_count_fusion.set_defaults(func=count_fusion)

    from celescope.fusion.run import run
    parser_run = subparsers_fusion_sub.add_parser(
        'run', help='run all steps', conflict_handler='resolve')
    get_opts_sample(parser_run, False)
    get_opts_barcode(parser_run, False)
    get_opts_cutadapt(parser_run, False)
    get_opts_STAR_fusion(parser_run, False)
    get_opts_count_fusion(parser_run, False)
    parser_run.set_defaults(func=run)

    # smk
    assay = 'smk'
    text = ASSAY_DICT[assay]
    subparsers_assay = subparsers.add_parser(
        assay, help=text, description=text)
    subparsers_assay_sub = subparsers_assay.add_subparsers()

    parser_tmp = subparsers_assay_sub.add_parser('sample')
    get_opts_sample(parser_tmp, True)
    parser_tmp.set_defaults(func=sample_info)

    parser_tmp = subparsers_assay_sub.add_parser('barcode')
    get_opts_barcode(parser_tmp, True)
    parser_tmp.set_defaults(func=barcode)

    parser_tmp = subparsers_assay_sub.add_parser('cutadapt')
    get_opts_cutadapt(parser_tmp, True)
    parser_tmp.set_defaults(func=cutadapt)

    from celescope.smk.mapping_smk import mapping_smk, get_opts_mapping_smk
    parser_tmp = subparsers_assay_sub.add_parser('mapping_smk')
    get_opts_mapping_smk(parser_tmp, True)
    parser_tmp.set_defaults(func=mapping_smk)

    from celescope.smk.count_smk import count_smk, get_opts_count_smk
    parser_tmp = subparsers_assay_sub.add_parser('count_smk')
    get_opts_count_smk(parser_tmp, True)
    parser_tmp.set_defaults(func=count_smk)

    from celescope.smk.analysis_smk import analysis_smk, get_opts_analysis_smk
    parser_tmp = subparsers_assay_sub.add_parser('analysis_smk')
    get_opts_analysis_smk(parser_tmp, True)
    parser_tmp.set_defaults(func=analysis_smk)

    from celescope.smk.run import run
    parser_tmp = subparsers_assay_sub.add_parser(
        'run', help='run all steps', conflict_handler='resolve')
    get_opts_sample(parser_tmp, False)
    get_opts_barcode(parser_tmp, False)
    get_opts_cutadapt(parser_tmp, False)
    get_opts_mapping_smk(parser_tmp, False)
    get_opts_count_smk(parser_tmp, False)
    get_opts_analysis_smk(parser_tmp, False)
    parser_tmp.set_defaults(func=run)

    # vdj
    assay = 'vdj'
    text = ASSAY_DICT[assay]
    subparsers_assay = subparsers.add_parser(
        assay, help=text, description=text)
    subparsers_assay_sub = subparsers_assay.add_subparsers()

    parser_tmp = subparsers_assay_sub.add_parser('sample')
    get_opts_sample(parser_tmp, True)
    parser_tmp.set_defaults(func=sample_info)

    parser_tmp = subparsers_assay_sub.add_parser('barcode')
    get_opts_barcode(parser_tmp, True)
    parser_tmp.set_defaults(func=barcode)

    parser_tmp = subparsers_assay_sub.add_parser('cutadapt')
    get_opts_cutadapt(parser_tmp, True)
    parser_tmp.set_defaults(func=cutadapt)

    from celescope.vdj.mapping_vdj import mapping_vdj, get_opts_mapping_vdj
    parser_tmp = subparsers_assay_sub.add_parser('mapping_vdj')
    get_opts_mapping_vdj(parser_tmp, True)
    parser_tmp.set_defaults(func=mapping_vdj)

    from celescope.vdj.count_vdj import count_vdj, get_opts_count_vdj
    parser_tmp = subparsers_assay_sub.add_parser('count_vdj')
    get_opts_count_vdj(parser_tmp, True)
    parser_tmp.set_defaults(func=count_vdj)

    from celescope.vdj.run import run
    parser_tmp = subparsers_assay_sub.add_parser(
        'run', help='run all steps', conflict_handler='resolve')
    get_opts_sample(parser_tmp, False)
    get_opts_barcode(parser_tmp, False)
    get_opts_cutadapt(parser_tmp, False)
    get_opts_mapping_vdj(parser_tmp, False)
    get_opts_count_vdj(parser_tmp, False)
    parser_tmp.set_defaults(func=run)

    # mut
    assay = 'mut'
    text = ASSAY_DICT[assay]
    subparsers_mut = subparsers.add_parser(
        assay, help=text, description=text)
    subparsers_mut_sub = subparsers_mut.add_subparsers()

    parser_sample = subparsers_mut_sub.add_parser('sample')
    get_opts_sample(parser_sample, True)
    parser_sample.set_defaults(func=sample_info)

    parser_barcode = subparsers_mut_sub.add_parser('barcode')
    get_opts_barcode(parser_barcode, True)
    parser_barcode.set_defaults(func=barcode)

    parser_cutadapt = subparsers_mut_sub.add_parser('cutadapt')
    get_opts_cutadapt(parser_cutadapt, True)
    parser_cutadapt.set_defaults(func=cutadapt)

    from celescope.mut.mapping_mut import mapping_mut, get_opts_mapping_mut
    parser_mapping_mut = subparsers_mut_sub.add_parser('mapping_mut')
    get_opts_mapping_mut(parser_mapping_mut, True)
    parser_mapping_mut.set_defaults(func=mapping_mut)

    from celescope.mut.count_mut import count_mut, get_opts_count_mut
    parser_count_mut = subparsers_mut_sub.add_parser('count_mut')
    get_opts_count_mut(parser_count_mut, True)
    parser_count_mut.set_defaults(func=count_mut)

    from celescope.fusion.run import run
    parser_run = subparsers_fusion_sub.add_parser(
        'run', help='run all steps', conflict_handler='resolve')
    get_opts_sample(parser_run, False)
    get_opts_barcode(parser_run, False)
    get_opts_cutadapt(parser_run, False)
    get_opts_STAR_fusion(parser_run, False)
    get_opts_count_fusion(parser_run, False)
    parser_run.set_defaults(func=run)

    # hla
    assay = 'hla'
    text = ASSAY_DICT[assay]
    subparsers_hla = subparsers.add_parser(
        assay, help=text, description=text)
    subparsers_hla_sub = subparsers_hla.add_subparsers()

    parser_sample = subparsers_hla_sub.add_parser('sample')
    get_opts_sample(parser_sample, True)
    parser_sample.set_defaults(func=sample_info)

    parser_barcode = subparsers_hla_sub.add_parser('barcode')
    get_opts_barcode(parser_barcode, True)
    parser_barcode.set_defaults(func=barcode)

    parser_cutadapt = subparsers_hla_sub.add_parser('cutadapt')
    get_opts_cutadapt(parser_cutadapt, True)
    parser_cutadapt.set_defaults(func=cutadapt)

    from celescope.hla.mapping_hla import mapping_hla, get_opts_mapping_hla
    parser_mapping_hla = subparsers_hla_sub.add_parser('mapping_hla')
    get_opts_mapping_hla(parser_mapping_hla, True)
    parser_mapping_hla.set_defaults(func=mapping_hla)

    '''
    from celescope.hla.count_hla import count_hla, get_opts_count_hla
    parser_count_hla = subparsers_hla_sub.add_parser('count_hla')
    get_opts_count_hla(parser_count_hla, True)
    parser_count_hla.set_defaults(func=count_hla)

    from celescope.hla.run import run
    parser_run = subparsers_fusion_sub.add_parser(
        'run', help='run all steps', conflict_handler='resolve')
    get_opts_sample(parser_run, False)
    get_opts_barcode(parser_run, False)
    get_opts_cutadapt(parser_run, False)
    get_opts_mapping_hla(parser_run, False)
    get_opts_count_hla(parser_run, False)
    parser_run.set_defaults(func=run)
    '''

    # capture_rna
    assay = 'capture_rna'
    text = ASSAY_DICT[assay]
    subparsers_capture_rna = subparsers.add_parser(assay, help=text, description=text)
    subparsers_capture_rna_sub = subparsers_capture_rna.add_subparsers()

    from celescope.tools.sample_info import sample_info, get_opts_sample
    parser_sample = subparsers_capture_rna_sub.add_parser('sample')
    get_opts_sample(parser_sample, True)
    parser_sample.set_defaults(func=sample_info)

    from celescope.tools.barcode import barcode, get_opts_barcode
    parser_barcode = subparsers_capture_rna_sub.add_parser('barcode')
    get_opts_barcode(parser_barcode, True)
    parser_barcode.set_defaults(func=barcode)

    from celescope.tools.cutadapt import cutadapt, get_opts_cutadapt
    parser_cutadapt = subparsers_capture_rna_sub.add_parser('cutadapt')
    get_opts_cutadapt(parser_cutadapt, True)
    parser_cutadapt.set_defaults(func=cutadapt)

    from celescope.tools.STAR import STAR, get_opts_STAR
    parser_STAR = subparsers_capture_rna_sub.add_parser('STAR')
    get_opts_STAR(parser_STAR, True)
    parser_STAR.set_defaults(func=STAR)

    from celescope.tools.featureCounts import featureCounts, get_opts_featureCounts
    parser_featureCounts = subparsers_capture_rna_sub.add_parser('featureCounts')
    get_opts_featureCounts(parser_featureCounts, True)
    parser_featureCounts.set_defaults(func=featureCounts)

    from celescope.capture_rna.count_capture_rna import count_capture_rna, get_opts_count_capture_rna
    parser_count_capture_rna = subparsers_capture_rna_sub.add_parser('count_capture_rna')
    get_opts_count_capture_rna(parser_count_capture_rna, True)
    parser_count_capture_rna.set_defaults(func=count_capture_rna)

    from celescope.tools.analysis import analysis, get_opts_analysis
    parser_analysis = subparsers_capture_rna_sub.add_parser('analysis')
    get_opts_analysis(parser_analysis, True)
    parser_analysis.set_defaults(func=analysis)

    # snp
    assay = 'snp'
    text = ASSAY_DICT[assay]
    subparsers_snp = subparsers.add_parser(assay, help=text, description=text)
    subparsers_snp = subparsers_snp.add_subparsers()

    from celescope.tools.sample_info import sample_info, get_opts_sample
    parser_sample = subparsers_snp.add_parser('sample')
    get_opts_sample(parser_sample, True)
    parser_sample.set_defaults(func=sample_info)

    from celescope.tools.barcode import barcode, get_opts_barcode
    parser_barcode = subparsers_snp.add_parser('barcode')
    get_opts_barcode(parser_barcode, True)
    parser_barcode.set_defaults(func=barcode)

    from celescope.tools.cutadapt import cutadapt, get_opts_cutadapt
    parser_cutadapt = subparsers_snp.add_parser('cutadapt')
    get_opts_cutadapt(parser_cutadapt, True)
    parser_cutadapt.set_defaults(func=cutadapt)

    from celescope.tools.STAR import STAR, get_opts_STAR
    parser_STAR = subparsers_snp.add_parser('STAR')
    get_opts_STAR(parser_STAR, True)
    parser_STAR.set_defaults(func=STAR)

    from celescope.tools.featureCounts import featureCounts, get_opts_featureCounts
    parser_featureCounts = subparsers_snp.add_parser('featureCounts')
    get_opts_featureCounts(parser_featureCounts, True)
    parser_featureCounts.set_defaults(func=featureCounts)

    from celescope.snp.snpCalling import snpCalling, get_opts_snpCalling
    parser_snpCalling = subparsers_snp.add_parser('snpCalling')
    get_opts_snpCalling(parser_snpCalling, True)
    parser_snpCalling.set_defaults(func=snpCalling)

    from celescope.snp.analysis_snp import analysis_snp, get_opts_analysis_snp
    parser_analysis_snp = subparsers_snp.add_parser('analysis_snp')
    get_opts_analysis_snp(parser_analysis_snp, True)
    parser_analysis_snp.set_defaults(func=analysis_snp)


    # tag
    assay = 'tag'
    text = ASSAY_DICT[assay]
    subparsers_assay = subparsers.add_parser(
        assay, help=text, description=text)
    subparsers_assay_sub = subparsers_assay.add_subparsers()

    parser_tmp = subparsers_assay_sub.add_parser('sample')
    get_opts_sample(parser_tmp, True)
    parser_tmp.set_defaults(func=sample_info)

    parser_tmp = subparsers_assay_sub.add_parser('barcode')
    get_opts_barcode(parser_tmp, True)
    parser_tmp.set_defaults(func=barcode)

    parser_tmp = subparsers_assay_sub.add_parser('cutadapt')
    get_opts_cutadapt(parser_tmp, True)
    parser_tmp.set_defaults(func=cutadapt)

    from celescope.tag.mapping_tag import mapping_tag, get_opts_mapping_tag
    parser_tmp = subparsers_assay_sub.add_parser('mapping_tag')
    get_opts_mapping_tag(parser_tmp, True)
    parser_tmp.set_defaults(func=mapping_tag)

    from celescope.tag.count_tag import count_tag, get_opts_count_tag
    parser_tmp = subparsers_assay_sub.add_parser('count_tag')
    get_opts_count_tag(parser_tmp, True)
    parser_tmp.set_defaults(func=count_tag)

    from celescope.tag.analysis_tag import analysis_tag, get_opts_analysis_tag
    parser_tmp = subparsers_assay_sub.add_parser('analysis_tag')
    get_opts_analysis_tag(parser_tmp, True)
    parser_tmp.set_defaults(func=analysis_tag)

    # citeseq
    assay = 'citeseq'
    text = ASSAY_DICT[assay]
    subparsers_assay = subparsers.add_parser(
        assay, help=text, description=text)
    subparsers_assay_sub = subparsers_assay.add_subparsers()

    parser_tmp = subparsers_assay_sub.add_parser('sample')
    get_opts_sample(parser_tmp, True)
    parser_tmp.set_defaults(func=sample_info)

    parser_tmp = subparsers_assay_sub.add_parser('barcode')
    get_opts_barcode(parser_tmp, True)
    parser_tmp.set_defaults(func=barcode)

    parser_tmp = subparsers_assay_sub.add_parser('cutadapt')
    get_opts_cutadapt(parser_tmp, True)
    parser_tmp.set_defaults(func=cutadapt)

    from celescope.tag.mapping_tag import mapping_tag, get_opts_mapping_tag
    parser_tmp = subparsers_assay_sub.add_parser('mapping_tag')
    get_opts_mapping_tag(parser_tmp, True)
    parser_tmp.set_defaults(func=mapping_tag)

    from celescope.citeseq.count_cite import count_cite, get_opts_count_cite
    parser_tmp = subparsers_assay_sub.add_parser('count_cite')
    get_opts_count_cite(parser_tmp, True)
    parser_tmp.set_defaults(func=count_cite)

    from celescope.citeseq.analysis_cite import analysis_cite, get_opts_analysis_cite
    parser_tmp = subparsers_assay_sub.add_parser('analysis_cite')
    get_opts_analysis_cite(parser_tmp, True)
    parser_tmp.set_defaults(func=analysis_cite)


    # tcr_fl
    assay = 'tcr_fl'
    text = ASSAY_DICT[assay]
    subparsers_assay = subparsers.add_parser(
        assay, help=text, description=text)
    subparsers_assay_sub = subparsers_assay.add_subparsers()

    parser_tmp = subparsers_assay_sub.add_parser('sample')
    get_opts_sample(parser_tmp, True)
    parser_tmp.set_defaults(func=sample_info)

    parser_tmp = subparsers_assay_sub.add_parser('barcode')
    get_opts_barcode(parser_tmp, True)
    parser_tmp.set_defaults(func=barcode)

    parser_tmp = subparsers_assay_sub.add_parser('cutadapt')
    get_opts_cutadapt(parser_tmp, True)
    parser_tmp.set_defaults(func=cutadapt)

    from celescope.tcr_fl.split_fq import split_fq, get_opts_split_fq
    parser_tmp = subparsers_assay_sub.add_parser('split_fq')
    get_opts_split_fq(parser_tmp, True)
    parser_tmp.set_defaults(func=split_fq)

    from celescope.tcr_fl.assemble import assemble, get_opts_assemble
    parser_tmp = subparsers_assay_sub.add_parser('assemble')
    get_opts_assemble(parser_tmp, True)
    parser_tmp.set_defaults(func=assemble)

    args = parser.parse_args()
    args.func(args)


if __name__ == '__main__':
    main()



import os
from collections import defaultdict
from concurrent.futures import ProcessPoolExecutor
import celescope
import pysam
import numpy as np
import pandas as pd
import logging
from celescope.tools.utils import format_number, log, read_barcode_file
from celescope.tools.utils import format_stat
from celescope.tools.utils import read_one_col, gene_convert, glob_genomeDir
from celescope.tools.report import reporter


@log
def split_bam(bam, barcodes, outdir, sample, gene_id_name_dic, min_query_length):
    '''
    input:
        bam: bam from feauturCounts
        barcodes: cell barcodes, set
        gene_id_name_dic: id name dic
        min_query_length: minimum query length

    ouput:
        bam_dict: assign reads to cell barcodes and UMI
        count_dict: UMI counts per cell
        index: assign index(1-based) to cells
    '''

    # init
    count_dict = defaultdict(dict)
    bam_dict = defaultdict(list)
    index_dict = defaultdict(dict)
    cells_dir = f'{outdir}/cells/'

    # read bam and split
    split_bam.logger.info('reading bam...')
    samfile = pysam.AlignmentFile(bam, "rb")
    header = samfile.header
    barcodes = list(barcodes)
    barcodes.sort()
    for read in samfile:
        attr = read.query_name.split('_')
        barcode = attr[0]
        umi = attr[1]
        if not read.has_tag('XT'):
            continue
        gene = read.get_tag('XT')
        query_length = read.infer_query_length()
        if (barcode in barcodes) and (gene in gene_id_name_dic) and (query_length >= min_query_length):
            gene_name = gene_id_name_dic[gene]
            read.set_tag(tag='GN', value=gene_name, value_type='Z')
            index = barcodes.index(barcode) + 1
            read.set_tag(tag='CL', value=f'CELL{index}', value_type='Z')

            # assign read to barcode
            bam_dict[barcode].append(read)

            # count
            if gene_name not in count_dict[barcode]:
                count_dict[barcode][gene_name] = {}
            if umi in count_dict[barcode][gene_name]:
                count_dict[barcode][gene_name][umi] += 1
            else:
                count_dict[barcode][gene_name][umi] = 1

    split_bam.logger.info('writing cell bam...')
    # write new bam
    index = 0
    for barcode in barcodes:
        # init
        index += 1
        index_dict[index]['barcode'] = barcode
        index_dict[index]['valid'] = False

        # out bam
        if barcode in bam_dict:
            cell_dir = f'{cells_dir}/cell{index}'
            cell_bam_file = f'{cell_dir}/cell{index}.bam'
            if not os.path.exists(cell_dir):
                os.makedirs(cell_dir)
            index_dict[index]['valid'] = True
            cell_bam = pysam.AlignmentFile(
                f'{cell_bam_file}', "wb", header=header)
            for read in bam_dict[barcode]:
                cell_bam.write(read)
            cell_bam.close()

    # out df_index
    df_index = pd.DataFrame(index_dict).T
    df_index.index.name = 'cell_index'
    index_file = f'{outdir}/{sample}_cell_index.tsv'
    df_index.to_csv(index_file, sep='\t')

    # out count_dict
    df_count = pd.DataFrame(columns=['barcode', 'gene', 'UMI', 'read_count'])
    for barcode in count_dict:
        for gene in count_dict[barcode]:
            for umi in count_dict[barcode][gene]:
                read_count = count_dict[barcode][gene][umi]
                df_count = df_count.append({
                    'barcode': barcode,
                    'gene': gene,
                    'UMI': umi,
                    'read_count': read_count,
                }, ignore_index=True)
    count_file = f'{outdir}/{sample}_count.tsv'
    df_count.to_csv(count_file, sep='\t', index=False)

    return index_file, count_file


def call_snp(index, outdir, fasta):

    logging.info(f'Processing Cell {index}..')
    bam = f'{outdir}/cells/cell{index}/cell{index}.bam'
    # sort
    sorted_bam = f'{outdir}/cells/cell{index}/cell{index}_sorted.bam'
    cmd_sort = (
        f'samtools sort {bam} -o {sorted_bam}'
    )
    os.system(cmd_sort)

    # mpileup
    out_bcf = f'{outdir}/cells/cell{index}/cell{index}.bcf'
    cmd_mpileup = (
        f'samtools mpileup -g -f '
        f'{fasta} '
        f'{sorted_bam} 2>/dev/null | '
        f'bcftools call -mv -Ob '
        f'-o {out_bcf} '
        f'>/dev/null 2>&1 '
    )
    os.system(cmd_mpileup)

    # view
    out_vcf = f'{outdir}/cells/cell{index}/cell{index}.vcf'
    cmd_view = (
        f'bcftools view {out_bcf} > {out_vcf}'
    )
    os.system(cmd_view)

    # norm
    norm_vcf = f'{outdir}/cells/cell{index}/cell{index}_norm.vcf'
    cmd_norm = (
        f'bcftools norm -d none '
        f'-f {fasta} '
        f'{out_vcf} '
        f'-o {norm_vcf} '
    )
    os.system(cmd_norm)


def read_index(index_file):
    df_index = pd.read_csv(index_file, sep='\t', index_col=0, dtype=object)
    df_valid = df_index[df_index['valid'] == 'True']
    return df_index, df_valid


@log
def call_all_snp(index_file, outdir, thread, fasta):
    all_res = []
    _df_index, df_valid = read_index(index_file)
    index_arg = df_valid.index
    outdir_arg = [outdir] * len(index_arg)
    fasta_arg = [fasta] * len(index_arg)
    with ProcessPoolExecutor(thread) as pool:
        for res in pool.map(call_snp, index_arg, outdir_arg, fasta_arg):
            all_res.append(res)


def process_vcf_header(line, sample):
    if line.startswith('##'):
        if line.find('samtools') != -1 or line.find('bcftools') != -1:
            return
        return line
    else:
        items = line.split('\t')
        items[-1] = sample
        new_line = '\t'.join(items) + '\n'
        return new_line


@log
def summary(index_file, count_file, outdir, sample):
    # init
    number = 0
    Number_of_Match_Cells_with_SNP = 0
    SNP_count_dict = defaultdict(int)
    coord_gene_dict = defaultdict(dict)

    # read index
    df_index, df_valid = read_index(index_file)

    # out vcf
    out_vcf = open(f'{outdir}/{sample}.vcf', 'wt')
    for index in df_valid.index:
        vcf_coords_dict = {}
        number += 1
        cell_vcf_file = f'{outdir}/cells/cell{index}/cell{index}_norm.vcf'
        # vcf coords
        with open(cell_vcf_file, 'rt') as f:
            for line in f:
                if line.startswith("#"):
                    # add vcf and bam header
                    if number == 1:
                        new_line = process_vcf_header(line, sample)
                        if new_line:
                            out_vcf.write(new_line)
                    continue
                if line:
                    items = line.split('\t')
                    items[7] += f';CELL={index}'
                    new_line = '\t'.join(items)
                    out_vcf.write(new_line)
                    chrom = str(items[0])
                    pos = int(items[1])
                    if chrom not in vcf_coords_dict:
                        vcf_coords_dict[chrom] = set([pos])
                    else:
                        vcf_coords_dict[chrom].add(pos)
                    SNP_count_dict[index] += 1

        # add bam header
        if number == 1:
            cell_bam_file = f'{outdir}/cells/cell{index}/cell{index}_sorted.bam'
            cell_bam = pysam.AlignmentFile(cell_bam_file, "rb")
            header = cell_bam.header
            out_bam = pysam.AlignmentFile(f'{outdir}/{sample}.bam', "wb", header=header)

        # add bam
        if len(vcf_coords_dict) > 0:
            Number_of_Match_Cells_with_SNP += 1
            cell_bam_file = f'{outdir}/cells/cell{index}/cell{index}_sorted.bam'
            cell_bam = pysam.AlignmentFile(cell_bam_file, "rb")
            for read in cell_bam:
                bam_ref = str(read.reference_name)
                gene_name = read.get_tag('GN')
                aligned_pairs = read.get_aligned_pairs()
                align_dict = {}
                for pair in aligned_pairs:
                    ref_pos = pair[1]
                    read_pos = pair[0]
                    if ref_pos:
                        align_dict[ref_pos] = read_pos
                if bam_ref in vcf_coords_dict.keys():
                    read_flag = False
                    for pos in vcf_coords_dict[bam_ref]:
                        if pos in align_dict:
                            read_flag = True
                            coord_gene_dict[bam_ref][pos] = gene_name
                    if read_flag:
                        out_bam.write(read)

    out_vcf.close()
    out_bam.close()
    pysam.sort("-o", f'{outdir}/{sample}_sorted.bam', f'{outdir}/{sample}.bam')
    cmd = f'samtools index {outdir}/{sample}_sorted.bam'
    os.system(cmd)

    # annotate vcf
    anno_vcf = open(f'{outdir}/{sample}_anno.vcf', 'wt')
    with open(f'{outdir}/{sample}.vcf', 'rt') as vcf:
        for line in vcf:
            if line.startswith('#'):
                anno_vcf.write(line)
                continue
            items = line.split('\t')
            chrom = str(items[0])
            pos = int(items[1])
            gene_name = coord_gene_dict[chrom][pos]
            items[7] += f';GENE={gene_name}'
            new_line = '\t'.join(items)
            anno_vcf.write(new_line)
    anno_vcf.close()

    # rm
    #os.remove(f'{outdir}/{sample}.vcf')
    #os.remove(f'{outdir}/{sample}.bam')

    # stat
    stats = pd.Series()
    n_match_cell = len(df_index.index)

    df_count = pd.read_csv(count_file, sep='\t')
    df_count_read = df_count.groupby('barcode').agg({'read_count':sum})
    read_total = sum(df_count_read['read_count'])
    Mean_Reads_per_Cell = round((read_total / n_match_cell), 2)
    stats = stats.append(pd.Series(
        Mean_Reads_per_Cell,
        index=['Mean Reads per Cell']
    ))
    df_count_UMI = df_count.groupby('barcode').agg({'UMI':'count'})
    UMI_total = sum(df_count_UMI['UMI'])
    Mean_UMIs_per_Cell = round((UMI_total / n_match_cell), 2)
    stats = stats.append(pd.Series(
        Mean_UMIs_per_Cell,
        index=['Mean UMIs per Cell']
    ))

    stats = stats.append(pd.Series(
        format_stat(Number_of_Match_Cells_with_SNP, n_match_cell),
        index=['Number of Cells with Variants']
    ))

    SNP_counts = list(SNP_count_dict.values())
    Mean_SNP_per_Cell = round(np.mean(SNP_counts), 3)
    stats = stats.append(pd.Series(
        Mean_SNP_per_Cell,
        index=['Mean Variants per Cell with Variants']
    ))

    stat_file = f'{outdir}/stat.txt'
    stats.to_csv(stat_file, sep=':', header=False)

    t = reporter(
        name='snpCalling', assay='snp', sample=sample,
        stat_file=stat_file, outdir=outdir + '/..')
    t.get_report()


@log
def convert(gene_list_file, gtf):
    gene_list_name, _count = read_one_col(gene_list_file)
    id_name = gene_convert(gtf)
    name_id = {}
    for id in id_name:
        name = id_name[id]
        name_id[name] = id
    gene_id_name_dic = {}
    for gene_name in gene_list_name:
        gene_id = name_id[gene_name]
        gene_id_name_dic[gene_id] = gene_name
    return gene_id_name_dic


@log
def snpCalling(args):

    sample = args.sample
    outdir = args.outdir
    thread = int(args.thread)
    match_dir = args.match_dir
    bam = args.bam
    genomeDir = args.genomeDir
    gene_list_file = args.gene_list
    min_query_length = args.min_query_length

    # process args
    barcodes, _nCell = read_barcode_file(match_dir)

    # check dir
    if not os.path.exists(outdir):
        os.system('mkdir -p %s' % (outdir))

    # get genome file
    _refFlat, gtf, fasta = glob_genomeDir(genomeDir, fa=True)

    # convert gene
    gene_id_name_dic = convert(gene_list_file, gtf)

    # split bam
    index_file, count_file = split_bam(
        bam, barcodes, outdir, sample, gene_id_name_dic, min_query_length)

    # snp
    call_all_snp(index_file, outdir, thread, fasta)

    # summary
    summary(index_file, count_file, outdir, sample)


def get_opts_snpCalling(parser, sub_program):
    if sub_program:
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument('--assay', help='assay', required=True)
        parser.add_argument("--thread", help='number of thread', default=1)
        parser.add_argument("--bam", help='featureCounts bam', required=True)
        parser.add_argument("--genomeDir", help='genomeDir', required=True)
    parser.add_argument(
        "--match_dir", help="match scRNA-Seq dir", required=True)
    parser.add_argument("--gene_list", help='gene_list', required=True)
    parser.add_argument("--min_query_length", help='minimum query length', default=35)



import os
from os import listdir
from os.path import isfile, join
from collections import defaultdict
from concurrent.futures import ProcessPoolExecutor
import pysam
import pandas as pd
from celescope.tools.utils import genDict, format_number, log, read_barcode_file

TRACER_PATH = '/SGRNJ/Public/Software/tracer/tracer'
CONF_PATH = '/SGRNJ01/RD_dir/pipeline_test/zhouyiqi/unittest/tcr_fl/20201103/tracer_SGR.conf'
CONDA = 'vdjpuzzle1'
CONDA_SUB = 'celescope_tracer'


def tracer_summarise(outdir):
    tracer_outdir = f'{outdir}/tracer'
    cmd = (
        f'source activate {CONDA_SUB}; '
        f'{TRACER_PATH} summarise '
        f'-c {CONF_PATH} '
        f'{tracer_outdir} '
    )
    os.system(cmd)


def tracer(fq, outdir):
    prefix = os.path.basename(fq).strip('.fq')
    cmd = (
        f'source activate {CONDA}; '
        f'{TRACER_PATH} assemble '
        f'--fragment_length 150 '
        f'--fragment_sd 5 '
        f'--single_end '
        f'--species Hsap '
        f'-c {CONF_PATH} '
        f'{fq} '
        f'{prefix} '
        f'{outdir}/tracer '
    )
    os.system(cmd)
    

@log
def run_assemble(sample, outdir, fastq_dir, thread):
    fqs = [join(fastq_dir, f) for f in listdir(fastq_dir) if isfile(join(fastq_dir, f))]
    outdirs = [outdir] * len(fqs)
    if not os.path.exists(f'{outdir}/tracer'):
        os.makedirs(f'{outdir}/tracer')
    
    all_res = []
    with ProcessPoolExecutor(thread) as pool:
        for res in pool.map(tracer, fqs, outdirs):
            all_res.append(res)
    
    tracer_summarise(outdir)


def assemble(args):
    thread = int(args.thread)
    fastq_dir = args.fastq_dir
    outdir = args.outdir
    sample = args.sample
    run_assemble(sample, outdir, fastq_dir, thread)


def get_opts_assemble(parser, sub_program):
    if sub_program:
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument("--fastq_dir", required=True)
        parser.add_argument('--assay', help='assay', required=True)
    parser.add_argument('--thread', help='thread', default=4)



from celescope.__init__ import __CONDA__
from celescope.rna.__init__ import __STEPS__, __ASSAY__
from celescope.tools.Multi import Multi


class Multi_rna(Multi):
    pass

def main():
    multi = Multi_rna(__ASSAY__, __STEPS__, __CONDA__)
    multi.run()

if __name__ == '__main__':
    main()



from celescope.__init__ import __CONDA__
from celescope.mut.__init__ import __STEPS__, __ASSAY__
from celescope.tools.Multi import Multi


class Multi_mut(Multi):
    
    def custome_args(self):
        self.STAR_args()
        self.parser.add_argument("--mut_file", help="mutation file", required=True)
        self.parser.add_argument("--shift_base", default=2)
        self.parser.add_argument('--indel_genomeDir', help='insertion or deletion STAR indexed genome directory', required=True)

    def read_custome_args(self):
        self.read_STAR_args()
        self.mut_file = self.args.mut_file
        self.shift_base = self.args.shift_base
        self.indel_genomeDir = self.args.indel_genomeDir    

    def mapping_mut(self, sample):
        step = 'mapping_mut'
        fq = f'{self.outdir_dic[sample]["cutadapt"]}/{sample}_clean_2.fq.gz'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step } '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--fq {fq} '
            f'--thread {self.thread} '
            f'--indel_genomeDir {self.indel_genomeDir} '
        )
        self.generate_other(cmd, step, sample, m=self.starMem, x=self.thread)

    def count_mut(self, sample):
        step = 'count_mut'
        bam = f'{self.outdir_dic[sample]["mapping_mut"]}/{sample}_Aligned.sortedByCoord.out.bam'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step } '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--bam {bam} '
            f'--mut_file {self.mut_file} '
            f'--match_dir {self.col4_dict[sample]} '
            f'shift_base {self.shift_base} '
        )
        self.generate_other(cmd, step, sample, m=8, x=1)


def main():
    multi = Multi_mut(__ASSAY__, __STEPS__, __CONDA__)
    multi.col4_default = None
    multi.run()

if __name__ == '__main__':
    main()




from celescope.vdj.__init__ import __STEPS__, __ASSAY__


def run(args):
    steps = __STEPS__
    args.assay = __ASSAY__
    sample = args.sample

    outdir_dic = {}
    index = 0
    for step in steps:
        outdir = f"{sample}/{index:02d}.{step}"
        outdir_dic.update({step: outdir})
        index += 1

    step = "sample"
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.tools.sample_info import sample_info
    sample_info(args)

    step = "barcode"
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.tools.barcode import barcode
    barcode(args)

    step = "cutadapt"
    args.outdir = f'{outdir_dic[step]}/'
    args.fq = f'{outdir_dic["barcode"]}/{sample}_2.fq.gz'
    from celescope.tools.cutadapt import cutadapt
    cutadapt(args)

    step = 'mapping_vdj'
    args.fq = f'{outdir_dic["cutadapt"]}/{sample}_clean_2.fq.gz'
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.vdj.mapping_vdj import mapping_vdj
    mapping_vdj(args)

    step = 'count_vdj'
    args.UMI_count_filter1_file = f'{outdir_dic["mapping_vdj"]}/{sample}_UMI_count_filtered1.tsv'
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.vdj.count_vdj import count_vdj
    count_vdj(args)



from celescope.tools.report import reporter
from celescope.tools.utils import format_number, read_barcode_file, log, format_stat
import matplotlib.pyplot as plt
import os
import pandas as pd
import logging
import numpy as np
import argparse
import glob
from collections import defaultdict
import matplotlib as mpl
mpl.use('Agg')


def get_UMI(row):
    return row.sum()


def get_UMI_min(df_cell_UMI, UMI_min):
    if UMI_min == "auto":
        UMI_min1 = np.percentile(df_cell_UMI.sum(axis=1), 5)
        UMI_min2 = np.median(df_cell_UMI.sum(axis=1)) / 10
        UMI_min = int(min(UMI_min1, UMI_min2))
        UMI_min = max(UMI_min, 1)
        return UMI_min
    else:
        return int(UMI_min)


def get_SNR(row, dim):
    row_sorted = sorted(row, reverse=True)
    noise = row_sorted[dim]
    signal = row_sorted[dim - 1]
    if noise == 0:
        return np.inf
    return float(signal) / noise


def get_SNR_min(df_cell_UMI, dim, SNR_min, UMI_min):
    UMIs = df_cell_UMI.apply(get_UMI, axis=1)
    df_valid_cell_UMI = df_cell_UMI[UMIs >= UMI_min]
    if SNR_min == "auto":
        SNRs = df_valid_cell_UMI.apply(get_SNR, dim=dim, axis=1)
        if np.median(SNRs) == np.inf:
            return 10
        return max(np.median(SNRs) / 10, 2)
    else:
        return float(SNR_min)


def tag_type(row, UMI_min, SNR_min, dim):
    SNR = get_SNR(row, dim)
    UMI = get_UMI(row)
    if UMI < UMI_min:
        return "Undetermined"
    if SNR < SNR_min:
        return "Multiplet"
    # get tag
    signal_tags = sorted(row.sort_values(ascending=False).index[0:dim])
    signal_tags_str = "_".join(signal_tags)
    return signal_tags_str


def write_and_plot(df, column_name, count_file, plot_file):
    df_count = df.groupby(["tag", column_name]).size().unstack()
    df_count.fillna(0, inplace=True)
    df_count.to_csv(count_file, sep="\t")
    df_percent = df_count / df_count.sum()
    df_plot = df_percent.stack().reset_index()
    df_plot.rename({0: "percent"}, axis=1, inplace=True)

    # plot
    colors = list(mpl.colors.cnames.keys())
    fig, ax = plt.subplots(figsize=(20, 10))
    types = df_plot["tag"].drop_duplicates()
    margin_bottom = np.zeros(len(df_plot[column_name].drop_duplicates()))

    for num, type in enumerate(types):
        values = list(df_plot.loc[df_plot["tag"] == type, "percent"])
        df_plot[df_plot['tag'] == type].plot.bar(
            x=column_name, y='percent', ax=ax, stacked=True,
            bottom=margin_bottom, label=type, color=colors[num * 3 + 1])
        margin_bottom += values
    ax.legend(loc='center left', bbox_to_anchor=(1, 0.5))
    plt.title("SMK tag fraction")
    fig.savefig(plot_file)


@log
def count_smk(args):

    read_file = args.read_file
    match_dir = args.match_dir
    tsne_file = glob.glob(f'{match_dir}/*analysis/*tsne_coord.tsv')[0]
    UMI_min = args.UMI_min
    SNR_min = args.SNR_min
    dim = int(args.dim)
    combine_cluster = args.combine_cluster
    outdir = args.outdir
    sample = args.sample
    assay = args.assay

    if not os.path.exists(outdir):
        os.system('mkdir -p %s' % (outdir))

    # stat_row
    stats = pd.Series()

    # process
    match_barcode, cell_total = read_barcode_file(match_dir)

    UMI_tag_file = f'{outdir}/{sample}_umi_tag.tsv'
    tsne_tag_file = f'{outdir}/{sample}_tsne_tag.tsv'
    cluster_count_file = f'{outdir}/{sample}_cluster_count.tsv'
    cluster_plot = f'{outdir}/{sample}_cluster_plot.pdf'
    if combine_cluster:
        combine_cluster_count_file = f'{outdir}/{sample}_combine_cluster_count.tsv'
        combine_cluster_plot = f'{outdir}/{sample}_combine_cluster_plot.pdf'

    df_read_count = pd.read_csv(read_file, sep="\t", index_col=0)
    mapped_read = df_read_count['read_count'].sum()

    # in cell
    df_read_count_in_cell = df_read_count[df_read_count.index.isin(match_barcode)]
    mapped_read_in_cell = int(df_read_count_in_cell['read_count'].sum())
    stats = stats.append(pd.Series(
        format_stat(mapped_read_in_cell, mapped_read),
        index=['Mapped Reads in Cells']
    ))

    # UMI
    df_UMI_in_cell = df_read_count_in_cell.reset_index().groupby([
        'barcode', 'SMK_barcode_name']).agg({'UMI': 'count'})
    df_UMI_in_cell = df_UMI_in_cell.reset_index()
    df_UMI_in_cell = df_UMI_in_cell.pivot(
        index='barcode', columns='SMK_barcode_name', values='UMI')
    df_cell = pd.DataFrame(index=match_barcode)
    df_UMI_cell = pd.merge(
        df_cell,
        df_UMI_in_cell,
        how="left",
        left_index=True,
        right_index=True)

    # fillna
    df_UMI_cell.fillna(0, inplace=True)
    df_UMI_cell = df_UMI_cell.astype(int)

    # UMI
    UMIs = df_UMI_cell.apply(sum, axis=1)
    median = round(np.median(UMIs), 2)
    mean = round(np.mean(UMIs), 2)
    stats = stats.append(pd.Series(
        str(median),
        index=['Median UMI per Cell']
    ))

    stats = stats.append(pd.Series(
        str(mean),
        index=['Mean UMI per Cell']
    ))

    UMI_min = get_UMI_min(df_UMI_cell, UMI_min)
    count_smk.logger.info(f'UMI_min: {UMI_min}')
    SNR_min = get_SNR_min(df_UMI_cell, dim, SNR_min, UMI_min)
    count_smk.logger.info(f'SNR_min: {SNR_min}')
    df_UMI_cell["tag"] = df_UMI_cell.apply(
        tag_type, UMI_min=UMI_min, SNR_min=SNR_min, dim=dim, axis=1)
    df_UMI_cell.to_csv(UMI_tag_file, sep="\t")

    df_tsne = pd.read_csv(tsne_file, sep="\t", index_col=0)
    df_tsne_tag = pd.merge(
        df_tsne,
        df_UMI_cell,
        how="left",
        left_index=True,
        right_index=True)

    if combine_cluster:
        df_combine_cluster = pd.read_csv(
            combine_cluster, sep="\t", header=None)
        df_combine_cluster.columns = ["cluster", "combine_cluster"]
        df_tsne_combine_cluster_tag = pd.merge(
            df_tsne_tag, df_combine_cluster,
            on=["cluster"], how="left", left_index=True).set_index(df_tsne_tag.index)
        df_tsne_combine_cluster_tag.to_csv(tsne_tag_file, sep="\t")
    else:
        df_tsne_tag.to_csv(tsne_tag_file, sep="\t")

    write_and_plot(
        df=df_tsne_tag, column_name="cluster", count_file=cluster_count_file,
        plot_file=cluster_plot
    )

    if combine_cluster:
        write_and_plot(
            df=df_tsne_combine_cluster_tag,
            column_name="combine_cluster",
            count_file=combine_cluster_count_file,
            plot_file=combine_cluster_plot
        )

    df_tag_count = df_UMI_cell["tag"].value_counts().reset_index()
    df_tag_count.columns = ["item", "count"]
    for index, row in df_tag_count.iterrows():
        stats = stats.append(pd.Series(
            format_stat(row['count'], cell_total),
            index=[row['item'] + ' Cells']
    ))
    stat_file = f'{outdir}/stat.txt'
    stats.to_csv(stat_file, sep=':', header=False)


    t = reporter(
        name='count_smk', assay=assay, sample=sample,
        stat_file=stat_file, outdir=outdir + '/..')
    t.get_report()


def get_opts_count_smk(parser, sub_program):
    parser.add_argument(
        "--UMI_min",
        help="cells have SMK_UMI>=UMI_min are considered as valid cell",
        default="auto")
    parser.add_argument("--dim", help="SMK tag dimension", default=1)
    parser.add_argument(
        "--SNR_min",
        help="minimum signal to noise ratio",
        default="auto")
    parser.add_argument(
        "--combine_cluster",
        help="conbine cluster tsv file",
        default=None)
    parser.add_argument("--match_dir", help="matched scRNA-Seq CeleScope directory path", required=True)
    if sub_program:
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument('--assay', help='assay', required=True)
        parser.add_argument("--read_file", help="SMK read count file")



#!/bin/env python
# coding=utf8

import os
import logging
logger1 = logging.getLogger(__name__)


def STAR_virus(args):

    logger1.info('align virus genome...')

    sample = args.sample
    outdir = args.outdir
    input_read = args.input_read
    virus_genomeDir = args.virus_genomeDir
    thread = args.thread
    outFilterMatchNmin = args.outFilterMatchNmin

    # check dir
    if not os.path.exists(outdir):
        os.system('mkdir -p %s' % (outdir))

    out_prefix = outdir + "/" + sample + "_virus_"
    out_BAM = out_prefix + "Aligned.sortedByCoord.out.bam"

    # host genome align
    cmd = "STAR \
 --genomeDir {genome} \
 --readFilesIn {input_read}\
 --outFilterMatchNmin 35\
 --outSAMtype BAM SortedByCoordinate\
 --runThreadN {runThreadN}\
 --limitBAMsortRAM 1933647594\
 --outFileNamePrefix {out_prefix}".format(genome=virus_genomeDir,
                                          input_read=input_read, runThreadN=thread, out_prefix=out_prefix)

    # add gz
    if input_read[len(input_read) - 2:] == "gz":
        cmd += ' --readFilesCommand zcat'

    logger1.info(cmd)
    os.system(cmd)
    logger1.info("align virus genome done.")

    cmd = "samtools index {out_BAM}".format(out_BAM=out_BAM)
    logger1.info(cmd)
    os.system(cmd)
    logger1.info("index done.")


def get_opts_STAR_virus(parser, sub_program):
    if sub_program:
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument("--input_read", required=True)
        parser.add_argument("--thread", help='STAR thread', default=1)
        parser.add_argument('--assay', help='assay', required=True)
    parser.add_argument(
        '--virus_genomeDir',
        help='virus genome dir',
        required=True)
    parser.add_argument("--outFilterMatchNmin", help='STAR outFilterMatchNmin', default=35)



#!/bin/env python
# coding=utf8

import os
import re
import logging
import subprocess
import glob
import sys
from celescope.tools.utils import format_number, log
from celescope.tools.utils import glob_genomeDir
from celescope.tools.report import reporter


def format_stat(log, samplename):
    #Assigned, Unassigned_NoFeatures, Unassigned_Ambiguity=(0, 0, 0)
    tmp_arr = []
    fh = open(log, 'r')
    with open(os.path.dirname(log) + '/stat.txt', 'w') as stat_fh:
        p1 = re.compile(r'Assigned.*?(\d+)', flags=re.S)
        p2 = re.compile(r'Unassigned_NoFeatures.*?(\d+)', flags=re.S)
        p3 = re.compile(r'Unassigned_Ambiguity.*?(\d+)', flags=re.S)
        for line in fh:
            if line.strip() == '':
                continue

            m1 = p1.search(line.strip())
            if m1:
                tmp_arr.append(int(m1.group(1)))

            m2 = p2.search(line)
            if m2:
                tmp_arr.append(int(m2.group(1)))

            m3 = p3.search(line)
            if m3:
                tmp_arr.append(int(m3.group(1)))

        total = sum(tmp_arr)
        tmp_arr = [
            '%s(%.2f%%)' %
            (format_number(n), (n + 0.0) / total * 100) for n in tmp_arr]
        for t, s in zip(['Assigned', 'Unassigned_NoFeatures',
                         'Unassigned_Ambiguity'], tmp_arr):
            stat_fh.write('%s: %s\n' % (t, s))
    fh.close()


@log
def featureCounts(args):

    # check
    _refFlat, gtf = glob_genomeDir(args.genomeDir)

    # check dir
    if not os.path.exists(args.outdir):
        os.mkdir(args.outdir)

    # run featureCounts
    outPrefix = args.outdir + '/' + args.sample
    cmd = ['featureCounts', 
        '-s', '1',
        '-a', gtf, '-o', outPrefix, '-R', 'BAM',
        '-T', str(args.thread), '-t', args.gtf_type, args.input]
    featureCounts.logger.info('%s' % (' '.join(cmd)))
    subprocess.check_call(cmd)

    subprocess.check_call(['which', 'samtools'])

    # sort by name:BC and umi
    featureCounts.logger.info('samtools sort ...!')
    bam_basename = os.path.basename(args.input)
    cmd = [
        'samtools',
        'sort',
        '-n',
        '-@',
        '3',
        '-o',
        outPrefix +
        '_name_sorted.bam',
        args.outdir +
        '/' +
        bam_basename +
        '.featureCounts.bam']
    featureCounts.logger.info('%s' % (' '.join(cmd)))
    subprocess.check_call(cmd)
    featureCounts.logger.info('samtools sort done.')

    format_stat(args.outdir + '/' + args.sample + '.summary', args.sample)
    t = reporter(
        name='featureCounts',
        assay=args.assay,
        sample=args.sample,
        stat_file=args.outdir +
        '/stat.txt',
        outdir=args.outdir +
        '/..')
    t.get_report()


def get_opts_featureCounts(parser, sub_program):

    parser.add_argument(
        '--gtf_type',
        help='Specify feature type in GTF annotation',
        default='exon')
    if sub_program:
        parser.add_argument('--genomeDir', required=True)
        parser.add_argument('--thread', default=1)
        parser.add_argument('--input', required=True)
        #parser.add_argument('--format', default='BAM')
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument('--assay', help='assay', required=True)



from celescope.__init__ import __CONDA__
from celescope.tag.__init__ import __STEPS__, __ASSAY__
from celescope.tools.Multi import Multi


class Multi_tag(Multi):
    def custome_args(self):
        self.parser.add_argument('--thread', help='thread', default=6)
        self.parser.add_argument(
            "--UMI_min",
            help="cells have tag_UMI>=UMI_min are considered as valid cell",
            default="auto")
        self.parser.add_argument("--dim", help="tag dimension", default=1)
        self.parser.add_argument(
            "--SNR_min",
            help="minimum signal to noise ratio",
            default="auto")
        self.parser.add_argument("--fq_pattern", help="tag read2 pattern", required=True)
        self.parser.add_argument("--linker_fasta", help="linker fasta")
        self.parser.add_argument("--barcode_fasta", help="barcode fasta", required=True)

    def read_custome_args(self):
        self.thread = self.args.thread
        self.UMI_min = self.args.UMI_min
        self.dim = self.args.dim
        self.SNR_min = self.args.SNR_min
        self.fq_pattern = self.args.fq_pattern
        self.linker_fasta = self.args.linker_fasta
        self.barcode_fasta = self.args.barcode_fasta

    def mapping_tag(self, sample):
        step = 'mapping_tag'
        fq = f'{self.outdir_dic[sample]["cutadapt"]}/{sample}_clean_2.fq.gz'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--fq {fq} '
            f'--fq_pattern {self.fq_pattern} '
            f'--barcode_fasta {self.barcode_fasta} '
            f'--linker_fasta {self.linker_fasta} '
        )
        self.generate_other(cmd, step, sample, m=5, x=1)

    def count_tag(self, sample):
        step = 'count_tag'
        read_count_file = f'{self.outdir_dic[sample]["mapping_tag"]}/{sample}_read_count.tsv'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--match_dir {self.col4_dict[sample]} '
            f'--read_count_file {read_count_file} '
            f'--dim {self.dim} '
            f'--UMI_min {self.UMI_min} '
            f'--SNR_min {self.SNR_min} '
        )
        self.generate_other(cmd, step, sample, m=5, x=1)


    def analysis_tag(self, sample):
        step = 'analysis_tag'
        tsne_tag_file = f'{self.outdir_dic[sample]["count_tag"]}/{sample}_tsne_tag.tsv'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--match_dir {self.col4_dict[sample]} '
            f'--tsne_tag_file {tsne_tag_file} '
        )
        self.generate_other(cmd, step, sample, m=5, x=1)



def main():
    multi = Multi_tag(__ASSAY__, __STEPS__, __CONDA__)
    multi.col4_default = None
    multi.run()

if __name__ == '__main__':
    main()




from celescope.tools.report import reporter
from celescope.tools.utils import format_number, log
from celescope.tools.utils import read_fasta, process_read, gen_stat
from celescope.tools.barcode import parse_pattern
import gzip
import os
import pandas as pd
import logging
import numpy as np
import sys
import argparse
import re
import glob
from collections import defaultdict
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


def genDict(dim=3):
    if dim == 1:
        return defaultdict(int)
    else:
        return defaultdict(lambda: genDict(dim - 1))


class smk_mapping:

    def __init__(
        self,
        SMK_read2,
        sample,
        SMK_pattern,
        SMK_linker,
        SMK_barcode,
        outdir,):

        # read args
        self.SMK_read2 = SMK_read2
        self.sample = sample
        self.SMK_pattern = SMK_pattern
        self.SMK_barcode = SMK_barcode
        self.SMK_linker = SMK_linker
        self.outdir = outdir

        # process
        self.SMK_barcode_dic, self.SMK_barcode_length = read_fasta(self.SMK_barcode, equal=True)
        self.SMK_linker_dic, self.SMK_linker_length = read_fasta(self.SMK_linker, equal=True)
        self.pattern_dict = parse_pattern(self.SMK_pattern)

        # check barcode length
        barcode1 = self.pattern_dict["C"][0]
        pattern_barcode_length = barcode1[1] - barcode1[0]
        if pattern_barcode_length != self.SMK_barcode_length:
            raise Exception(
                f'''SMK barcode fasta length {self.SMK_barcode_length} 
                != pattern barcode length {pattern_barcode_length}'''
            )

        self.res_dic = genDict()
        self.res_sum_dic = genDict(dim=2)

        self.match_barcode = []
        self.read_count_file = f'{outdir}/{sample}_read_count.tsv'
        self.UMI_count_file = f'{outdir}/{sample}_UMI_count.tsv'
        self.stat_file = self.outdir + "/stat.txt"

        if not os.path.exists(outdir):
            os.system('mkdir -p %s' % outdir)

    def read_to_dic(self):

        self.res_dic, self.metrics = process_read(
            self.SMK_read2,
            self.pattern_dict,
            self.SMK_barcode_dic,
            self.SMK_linker_dic,
            self.SMK_barcode_length,
            self.SMK_linker_length,
        )

        # write dic to pandas df
        rows = []
        for barcode in self.res_dic:
            for SMK_barcode_name in self.res_dic[barcode]:
                for umi in self.res_dic[barcode][SMK_barcode_name]:
                    rows.append([barcode, SMK_barcode_name, umi,
                                 self.res_dic[barcode][SMK_barcode_name][umi]])
        df_read_count = pd.DataFrame(rows)
        df_read_count.rename(
            columns={
                0: "barcode",
                1: "SMK_barcode_name",
                2: "UMI",
                3: "read_count"},
            inplace=True)
        df_read_count.to_csv(
            self.read_count_file, sep="\t", index=False)

        # stat
        stat = pd.DataFrame(
            {
                "item": [
                    "Reads Mapped",
                    "Reads Unmapped too Short",
                    "Reads Unmapped Invalid Linker",
                    "Reads Unmapped Invalid Barcode",
                ],
                "count": [
                    self.metrics["Reads Mapped"],
                    self.metrics["Reads Unmapped too Short"],
                    self.metrics["Reads Unmapped Invalid Linker"],
                    self.metrics["Reads Unmapped Invalid Barcode"],
                ],
            }, 
            columns=["item", "count"]
        )
        stat['total_count'] = self.metrics["Total Reads"]
        gen_stat(stat, self.stat_file)

    def tag_count(self):
        for barcode in self.res_dic:
            for SMK_barcode_name in self.res_dic[barcode]:
                self.res_sum_dic[barcode][SMK_barcode_name] = len(
                    self.res_dic[barcode][SMK_barcode_name])

        df_umi_count = pd.DataFrame(self.res_sum_dic)
        df_umi_count = df_umi_count.T
        df_umi_count.fillna(0, inplace=True)
        df_umi_count = df_umi_count.astype(int)
        df_umi_count.to_csv(self.UMI_count_file, sep="\t")

    @log
    def run(self):
        self.read_to_dic()
        self.tag_count()
        t = reporter(
            name='mapping_smk', assay="smk", sample=self.sample,
            stat_file=self.stat_file, outdir=self.outdir + '/..')
        t.get_report()


def get_opts_mapping_smk(parser, sub_program):
    parser.add_argument("--SMK_pattern", help="SMK read2 pattern")
    parser.add_argument("--SMK_linker", help="SMK read2 linker fasta path")
    parser.add_argument("--SMK_barcode", help="SMK read2 barcode fasta path ")
    if sub_program:
        parser.add_argument(
            "--SMK_read2",
            help="SMK clean read2",
            required=True)
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument('--assay', help='assay', required=True)


def mapping_smk(args):
    sample = args.sample
    outdir = args.outdir
    SMK_pattern = args.SMK_pattern
    SMK_linker = args.SMK_linker
    SMK_barcode = args.SMK_barcode
    SMK_read2 = args.SMK_read2

    s = smk_mapping(
        SMK_read2,
        sample,
        SMK_pattern,
        SMK_linker,
        SMK_barcode,
        outdir)
    s.run()



import os
import sys
import json
import logging
import re
import numpy as np
import pandas as pd
import glob
import subprocess
import configparser
from scipy.io import mmwrite
from scipy.sparse import csr_matrix
from celescope.tools.report import reporter
from celescope.tools.utils import glob_genomeDir, log, parse_annovar
from celescope.tools.utils import cluster_tsne_list, marker_table, report_prepare, parse_vcf
from .snpCalling import read_index


class analysis_variant():

    def __init__(self, outdir, sample, match_dir, vcf_file, index_file, assay, annovar_config):
        self.outdir = outdir
        self.sample = sample
        self.match_dir = match_dir
        self.vcf_file = vcf_file
        self.index_file = index_file 
        self.assay = assay  
        self.annovar_config = annovar_config
        if not os.path.exists(outdir):
            os.system('mkdir -p %s' % (outdir))

        #
        tsne_df_file = glob.glob(f'{match_dir}/*analysis*/*tsne_coord.tsv')[0]
        #marker_df_file = glob.glob(f'{match_dir}/*analysis*/*markers.tsv')[0]
        self.tsne_df = pd.read_csv(tsne_df_file, sep="\t")
        self.tsne_df.rename(columns={"Unnamed: 0": "barcode"}, inplace=True)

        self.cluster_tsne = cluster_tsne_list(self.tsne_df)

    def get_table(self):
        """
        Input:
            self.df_vcf
            self.index_file
            self.tsne_df
        Set Value:
            self.df_table_full
            self.df_table
            self.df_count
        """
        # df_vcf_count
        self.df_vcf.index = [int(index) for index in list(self.df_vcf['CELL'])]
        df_vcf_count = self.df_vcf.groupby(['CHROM','POS','ALLELES','GENE']).agg({
            'CELL':'count'}).reset_index().sort_values(['CELL'],ascending=False)

        # vcf_tsne
        df_index, df_valid = read_index(self.index_file)
        df_vcf_barcode = pd.merge(self.df_vcf, df_valid, left_index=True, right_index=True, how="left")
        df_vcf_tsne = pd.merge(df_vcf_barcode, self.tsne_df, on="barcode", how="left")

        # df_table
        df_vcf_count.rename(columns={'CELL':'NCELL'}, inplace=True)
        df_vcf_tsne.rename(columns={'CELL':'INDEX'}, inplace=True)
        df_vcf_tsne = df_vcf_tsne.astype({'CHROM':'object', 'POS':'int64'})
        df_vcf_count = df_vcf_count.astype({'CHROM':'object', 'POS':'int64'})
        df_table = pd.merge(df_vcf_tsne, df_vcf_count, on=['CHROM',"POS","ALLELES",'GENE'], how='left')
        self.df_table_full = df_table
        df_table_full_file = f'{self.outdir}/{self.sample}_variant_table.tsv'
        self.df_table_full.to_csv(df_table_full_file, sep='\t')
        cols = ['CHROM', 'POS', 'GENE', 'ALLELES', 'MRNA', 'PROTEIN', 'COSMIC', 'DP', 'GT', 'INDEX', 'cluster', 'NCELL']
        df_table = df_table.loc[:, cols]
        df_table.rename(columns={'cluster':'CLUSTER'}, inplace=True)
        self.df_table = df_table

        # vcf_tsne_all
        df_vcf_tsne_count = df_vcf_tsne.groupby(['barcode','INDEX']).agg({'ALLELES':'count'})
        df_vcf_tsne_all = pd.merge(self.tsne_df, df_vcf_tsne_count, on='barcode', how='left')
        df_index = df_index.reset_index()
        df_vcf_tsne_all = pd.merge(df_vcf_tsne_all, df_index, on='barcode', how='left')
        df_vcf_tsne_all.fillna(0, inplace=True)
        tSNE_1 = list(df_vcf_tsne_all.tSNE_1)
        tSNE_2 = list(df_vcf_tsne_all.tSNE_2)
        def return_text(row):
            text = f'Variants:{str(int(row["ALLELES"]))}<br>Cell_Index:{row["cell_index"]}'
            return text
        text = list(df_vcf_tsne_all.apply(return_text, axis=1))
        value = list(df_vcf_tsne_all.ALLELES)
        title = 't-SNE plot Colored by Cell Variant Counts'
        self.count_tsne = {"tSNE_1": tSNE_1, "tSNE_2": tSNE_2, "text": text, 'value':value, 'title':title}

    def get_variant_table(self):
        """
        return html code
        """
        self.variant_table = {}
        self.variant_table['title'] = 'Variant Table'
        pd.set_option('display.max_colwidth', -1)
        self.variant_table['table'] = self.df_table.to_html(
            escape=False,
            index=False,
            table_id="variant_table",
            justify="center")

    def report(self):
        t = reporter(
        name='analysis_snp',
        assay=self.assay,
        sample=self.sample,
        outdir=self.outdir + '/..')
        t.get_report()

    @log
    def annovar(self):

        # config
        config = configparser.ConfigParser()
        config.read(self.annovar_config)
        section = config['ANNOVAR']
        dir = section['dir']
        db = section['db']
        buildver = section['buildver']
        protocol = section['protocol']
        operation = section['operation']

        # convert
        input_file = f'{self.outdir}/{self.sample}.input'
        cmd = (
            f'perl {dir}/convert2annovar.pl '
            f'-format vcf4 '
            #f'--includeinfo '
            f'{self.vcf_file} > {input_file}'
        )
        subprocess.check_call(cmd, shell=True)

        # annotate
        cmd = (
            f'perl {dir}/table_annovar.pl '
            f'{input_file} '
            f'{db} '
            f'-buildver {buildver} '
            f'-protocol {protocol} '
            f'-operation {operation} '
            f'-out {self.outdir}/{self.sample} '
            f'--otherinfo '
        )
        analysis_variant.annovar.logger.info(cmd)
        subprocess.check_call(cmd, shell=True)

        # df
        annovar_file = f'{self.outdir}/{self.sample}.{buildver}_multianno.txt'
        self.df_annovar = parse_annovar(annovar_file)

    def get_df_vcf(self):
        self.df_vcf = parse_vcf(self.vcf_file)

    def run(self):
        self.get_df_vcf()
        if self.annovar_config:
            self.annovar()
            self.df_vcf = pd.concat((self.df_vcf, self.df_annovar), axis=1)
        self.get_table()
        self.get_variant_table()
        report_prepare(
            self.outdir,
            cluster_tsne=self.cluster_tsne, 
            count_tsne=self.count_tsne, 
            variant_table=self.variant_table
        )
        self.report()

@log
def analysis_snp(args):
    step_snp = analysis_variant(
        args.outdir,
        args.sample,
        args.match_dir,
        args.vcf_anno,
        args.index_file,
        args.assay,
        args.annovar_config,
    )
    step_snp.run()

def get_opts_analysis_snp(parser, sub_program):
    parser.add_argument('--annovar_config', help='annovar soft config file')
    if sub_program:
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument('--match_dir', help='match_dir', required=True)
        parser.add_argument('--vcf_anno', help='annotated vcf file', required=True)
        parser.add_argument('--index_file', help='index_file', required=True)
        parser.add_argument('--assay', help='assay', required=True)


import os
from celescope.tools.utils import log
from celescope.tools.STAR import Step_mapping


@log
def STAR_fusion(args):
    s = Step_mapping(
        args.sample,
        args.outdir,
        args.assay,
        args.thread,
        args.input_read,
        args.genomeDir,
        outFilterMatchNmin = args.outFilterMatchNmin,
    )
    s.STAR()


def get_opts_STAR_fusion(parser, sub_program):
    if sub_program:
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument("--input_read", required=True)
        parser.add_argument('--assay', help='assay', required=True)
    parser.add_argument(
        '--genomeDir',
        help='fusion gene STAR index genome directory',
        required=True)
    parser.add_argument("--thread", help='STAR thread', default=1)
    parser.add_argument("--outFilterMatchNmin", help='STAR outFilterMatchNmin', default=35)



from celescope.__init__ import __CONDA__
from celescope.tcr_fl.__init__ import __STEPS__, __ASSAY__
from celescope.tools.Multi import Multi


class Multi_tcr_fl(Multi):
    def custome_args(self):
        self.parser.add_argument('--thread', help='thread', default=4)
        self.parser.add_argument("--nCell", help="select top N cell")

    def read_custome_args(self):
        self.thread = self.args.thread
        self.nCell = self.args.nCell

    def split_fq(self, sample):
        step = 'split_fq'
        fq = f'{self.outdir_dic[sample]["cutadapt"]}/{sample}_clean_2.fq.gz'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--fq {fq} '
            f'--nCell {self.nCell} '
            f'--match_dir {self.col4_dict[sample]} '
        )
        self.process_cmd(cmd, step, sample, m=5, x=1)

    def assemble(self, sample):
        step = 'assemble'
        fastq_dir = f'{self.outdir_dic[sample]["split_fq"]}/fastq'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--fastq_dir {fastq_dir} '
            f'--thread {self.thread} '
        )
        self.process_cmd(cmd, step, sample, m=4 * int(self.thread), x=self.thread)


def main():
    multi = Multi_tcr_fl(__ASSAY__, __STEPS__, __CONDA__)
    multi.col4_default = None
    multi.run()

if __name__ == '__main__':
    main()




#!/bin/env python
# coding=utf8

import os
import pandas as pd
import sys
import logging
from celescope.__init__ import __VERSION__, ASSAY_DICT
from celescope.tools.utils import log
from celescope.tools.report import reporter
from celescope.tools.__init__ import __PATTERN_DICT__
from .Chemistry import Chemistry


@log
def sample_info(args):

    sample = args.sample
    ASSAY = ASSAY_DICT[args.assay]
    version = __VERSION__
    outdir = args.outdir
    chemistry = args.chemistry

    # get chemistry
    if chemistry == 'auto':
        fq1 = args.fq1
        fq1_file0 = fq1.split(',')[0]
        ch = Chemistry(fq1_file0)
        chemistry = ch.get_chemistry()
    else:
        chemistry = args.chemistry

    if not os.path.exists(outdir):
        os.system('mkdir -p %s' % outdir)

    stat = pd.DataFrame({
        "item": ["Sample ID", "Assay", "Chemistry", "Software Version"],
        "count": [sample, ASSAY, chemistry, version],
        },
        columns=["item", "count"]
    )
    stat_file = outdir + "/stat.txt"
    stat.to_csv(stat_file, sep=":", header=None, index=False)

    t = reporter(
        name='sample',
        assay=args.assay,
        sample=args.sample,
        stat_file=stat_file,
        outdir=outdir + '/..')
    t.get_report()
    return chemistry


def get_opts_sample(parser, sub_program):
    if sub_program:
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument('--assay', help='assay', required=True)
        parser.add_argument('--fq1', help='read1 fq file')
    parser.add_argument('--chemistry', choices=__PATTERN_DICT__.keys(), help='chemistry version', default='auto')
    



#!/bin/env python
# coding=utf8

import os
import glob
import sys
import argparse
import re
import logging
from collections import defaultdict
from celescope.__init__ import __CONDA__
from celescope.rna_virus.__init__ import __STEPS__, __ASSAY__
from celescope.tools.utils import merge_report, generate_sjm
from celescope.tools.utils import parse_map_col4, multi_opts, link_data


def main():

    # init
    assay = __ASSAY__
    steps = __STEPS__
    conda = __CONDA__
    app = 'celescope'

    # parser
    parser = multi_opts(assay)
    parser.add_argument('--starMem', help='starMem', default=30)
    parser.add_argument('--genomeDir', help='genome index dir', required=True)
    parser.add_argument(
        '--gtf_type',
        help='Specify attribute type in GTF annotation, default=exon',
        default='exon')
    parser.add_argument('--thread', help='thread', default=6)
    parser.add_argument(
        '--virus_genomeDir',
        help='virus_genomeDir',
        required=True)
    args = parser.parse_args()

    fq_dict, cells_dict = parse_map_col4(args.mapfile, 'auto')

    # read args
    outdir = args.outdir
    chemistry = args.chemistry
    pattern = args.pattern
    whitelist = args.whitelist
    linker = args.linker
    lowQual = args.lowQual
    lowNum = args.lowNum
    mod = args.mod
    rm_files = args.rm_files

    # parse mapfile
    fq_dict, cells_dict = parse_map_col4(args.mapfile, "auto")

    # link
    link_data(outdir, fq_dict)

    # custom args
    thread = args.thread
    genomeDir = args.genomeDir
    starMem = args.starMem
    gtf_type = args.gtf_type
    virus_genomeDir = args.virus_genomeDir

    # mk log dir
    logdir = outdir + '/log'
    os.system('mkdir -p %s' % (logdir))

    # script init
    sjm_cmd = 'log_dir %s\n' % (logdir)
    sjm_order = ''
    shell_dict = defaultdict(str)

    # outdir dict
    for sample in fq_dict:
        outdir_dic = {}
        index = 0
        for step in steps:
            step_outdir = f"{outdir}/{sample}/{index:02d}.{step}"
            outdir_dic.update({step: step_outdir})
            index += 1

        # sample
        step = "sample"
        cmd = (
            f'{app} {assay} {step} '
            f'--chemistry {chemistry} '
            f'--sample {sample} --outdir {outdir_dic[step]} --assay {assay} '
        )
        sjm_cmd += generate_sjm(cmd, f'{step}_{sample}', conda)
        shell_dict[sample] += cmd + '\n'
        last_step = step

        # barcode
        arr = fq_dict[sample]
        step = "barcode"
        cmd = (
            f'{app} {assay} {step} '
            f'--fq1 {arr[0]} --fq2 {arr[1]} --chemistry {chemistry} '
            f'--pattern {pattern} --whitelist {whitelist} --linker {linker} '
            f'--sample {sample} --lowQual {lowQual} --thread {thread} '
            f'--lowNum {lowNum} --outdir {outdir_dic[step]} --assay {assay} '

        )
        sjm_cmd += generate_sjm(cmd, f'{step}_{sample}', conda, m=5, x=thread)
        sjm_order += f'order {step}_{sample} after {last_step}_{sample}\n'
        shell_dict[sample] += cmd + '\n'
        last_step = step

        # adapt
        step = "cutadapt"
        fq = f'{outdir_dic["barcode"]}/{sample}_2.fq.gz'
        cmd = (
            f'{app} {assay} {step} '
            f'--fq {fq} --sample {sample} --outdir '
            f'{outdir_dic[step]} --assay {assay} '
        )
        sjm_cmd += generate_sjm(cmd, f'{step}_{sample}', conda, m=5, x=1)
        sjm_order += f'order {step}_{sample} after {last_step}_{sample}\n'
        shell_dict[sample] += cmd + '\n'
        last_step = step

        # STAR
        step = 'STAR'
        fq = f'{outdir_dic["cutadapt"]}/{sample}_clean_2.fq.gz'
        cmd = (
            f'{app} {assay} {step} '
            f'--fq {fq} --sample {sample} '
            f'--outdir {outdir_dic[step]} --assay {assay} '
            f'--genomeDir {genomeDir} --thread {thread} '
            f'--out_unmapped '
        )
        sjm_cmd += generate_sjm(cmd, f'{step}_{sample}', conda, m=starMem, x=thread)
        sjm_order += f'order {step}_{sample} after {last_step}_{sample}\n'
        shell_dict[sample] += cmd + '\n'
        last_step = step

        # STAR_virus
        step = 'STAR_virus'
        input_read = f'{outdir_dic["STAR"]}/{sample}_Unmapped.out.mate1'
        cmd = (
            f'{app} {assay} {step} '
            f'--sample {sample} --outdir {outdir_dic[step]} --assay {assay} '
            f'--input_read {input_read} '
            f'--virus_genomeDir {virus_genomeDir} '
            f'--thread {thread} '
        )
        sjm_cmd += generate_sjm(cmd, f'{step}_{sample}', conda, m=starMem, x=thread)
        sjm_order += f'order {step}_{sample} after {last_step}_{sample}\n'
        shell_dict[sample] += cmd + '\n'
        last_step = step

        # featureCounts
        step = 'featureCounts'
        input = f'{outdir_dic["STAR"]}/{sample}_Aligned.sortedByCoord.out.bam'
        cmd = (
            f'{app} {assay} {step} '
            f'--input {input} --gtf_type {gtf_type} '
            f'--sample {sample} --thread {thread} --outdir {outdir_dic[step]} '
            f'--genomeDir {genomeDir} --assay {assay} '
        )
        sjm_cmd += generate_sjm(cmd, f'{step}_{sample}', conda, m=8, x=thread)
        sjm_order += f'order {step}_{sample} after {last_step}_{sample}\n'
        shell_dict[sample] += cmd + '\n'
        last_step = step

        # count
        step = 'count'
        bam = f'{outdir_dic["featureCounts"]}/{sample}_name_sorted.bam'
        cmd = (
            f'{app} {assay} {step} '
            f'--bam {bam} --sample {sample} --cells {cells_dict[sample]} '
            f'--outdir {outdir_dic[step]} --assay {assay} '
            f'--genomeDir {genomeDir}'
        )
        sjm_cmd += generate_sjm(cmd, f'{step}_{sample}', conda, m=8, x=thread)
        sjm_order += f'order {step}_{sample} after {last_step}_{sample}\n'
        shell_dict[sample] += cmd + '\n'
        last_step = step

        # count_virus
        step = 'count_virus'
        virus_bam = f'{outdir_dic["STAR_virus"]}/{sample}_virus_Aligned.sortedByCoord.out.bam'
        barcode_file = f'{outdir_dic["count"]}/{sample}_matrix_10X/barcodes.tsv'
        cmd = (
            f'{app} {assay} {step} '
            f'--sample {sample} --outdir {outdir_dic[step]} --assay {assay} '
            f'--virus_bam {virus_bam} '
            f'--barcode_file {barcode_file} '
        )
        sjm_cmd += generate_sjm(cmd, f'{step}_{sample}', conda, m=20, x=thread)
        sjm_order += f'order {step}_{sample} after {last_step}_{sample}\n'
        shell_dict[sample] += cmd + '\n'
        last_step = step

        # analysis_rna_virus
        step = 'analysis_rna_virus'
        matrix_file = f'{outdir_dic["count"]}/{sample}_matrix.tsv.gz'
        virus_file = f'{outdir_dic["count_virus"]}/{sample}_virus_UMI_count.tsv'
        cmd = (
            f'{app} {assay} {step} '
            f'--sample {sample} --outdir {outdir_dic[step]} --assay {assay} '
            f'--matrix_file {matrix_file} '
            f'--virus_file {virus_file} '
        )
        sjm_cmd += generate_sjm(cmd, f'{step}_{sample}', conda, m=15, x=1)
        sjm_order += f'order {step}_{sample} after {last_step}_{sample}\n'
        shell_dict[sample] += cmd + '\n'
        last_step = step

    # merged report
    if mod == 'sjm':
        step = 'merge_report'
        merge_report(
            fq_dict,
            steps,
            last_step,
            sjm_cmd,
            sjm_order,
            logdir,
            conda,
            outdir,
            rm_files,
        )
    if mod == 'shell':
        os.system('mkdir -p ./shell/')
        for sample in shell_dict:
            with open(f'./shell/{sample}.sh', 'w') as f:
                f.write(shell_dict[sample])


if __name__ == '__main__':
    main()



from celescope.tools.report import reporter
from celescope.tools.utils import format_number, read_barcode_file, log, format_stat
import matplotlib.pyplot as plt
import os
import pandas as pd
import logging
import numpy as np
import argparse
import glob
from collections import defaultdict
import matplotlib as mpl
mpl.use('Agg')


class Count_tag():

    def __init__(
        self,
        sample,
        outdir,
        assay,
        read_count_file,
        match_dir,
        UMI_min,
        SNR_min,
        combine_cluster,
        dim,
        ):
        self.sample = sample
        self.outdir = outdir
        self.assay = assay
        self.read_count_file = read_count_file
        self.match_dir = match_dir
        self.UMI_min = UMI_min
        self.SNR_min = SNR_min
        self.combine_cluster = combine_cluster
        self.dim = int(dim)
        self.match_barcode, self.cell_total = read_barcode_file(match_dir)
        self.df_read_count = pd.read_csv(read_count_file, sep="\t", index_col=0)
        self.tsne_file = glob.glob(f'{match_dir}/*analysis/*tsne_coord.tsv')[0]
        self.no_noise = False
        if not os.path.exists(outdir):
            os.system('mkdir -p %s' % outdir)



    @staticmethod
    def get_UMI(row):
        return row.sum()

    @staticmethod
    def get_UMI_min(df_cell_UMI, UMI_min):
        if UMI_min == "auto":
            UMI_min1 = np.percentile(df_cell_UMI.sum(axis=1), 5)
            UMI_min2 = np.median(df_cell_UMI.sum(axis=1)) / 10
            UMI_min = int(min(UMI_min1, UMI_min2))
            UMI_min = max(UMI_min, 1)
            return UMI_min
        else:
            return int(UMI_min)

    @staticmethod
    def get_SNR(row, dim):
        row_sorted = sorted(row, reverse=True)            
        noise = row_sorted[dim]
        signal = row_sorted[dim - 1]
        if noise == 0:
            return np.inf
        return float(signal) / noise

    def get_SNR_min(self, df_cell_UMI, dim, SNR_min, UMI_min):
        UMIs = df_cell_UMI.apply(Count_tag.get_UMI, axis=1)
        df_valid_cell_UMI = df_cell_UMI[UMIs >= UMI_min]
        if SNR_min == "auto":
            # no noise
            if df_valid_cell_UMI.shape[1] <= dim:
                logging.warning('*** No NOISE FOUND! ***')
                self.no_noise = True
                return 0
            SNRs = df_valid_cell_UMI.apply(Count_tag.get_SNR, dim=dim, axis=1)
            if np.median(SNRs) == np.inf:
                return 10
            return max(np.median(SNRs) / 10, 2)
        else:
            return float(SNR_min)

    @staticmethod
    def tag_type(row, UMI_min, SNR_min, dim, no_noise=False):
        if no_noise:
            SNR = 1
        else:
            SNR = Count_tag.get_SNR(row, dim)
        UMI = Count_tag.get_UMI(row)
        if UMI < UMI_min:
            return "Undetermined"
        if SNR < SNR_min:
            return "Multiplet"
        # get tag
        signal_tags = sorted(row.sort_values(ascending=False).index[0:dim])
        signal_tags_str = "_".join(signal_tags)
        return signal_tags_str


    def write_and_plot(self, df, column_name, count_file, plot_file):
        df_count = df.groupby(["tag", column_name]).size().unstack()
        df_count.fillna(0, inplace=True)
        df_count.to_csv(count_file, sep="\t")
        df_percent = df_count / df_count.sum()
        df_plot = df_percent.stack().reset_index()
        df_plot.rename({0: "percent"}, axis=1, inplace=True)

        # plot
        colors = list(mpl.colors.cnames.keys())
        fig, ax = plt.subplots(figsize=(20, 10))
        types = df_plot["tag"].drop_duplicates()
        margin_bottom = np.zeros(len(df_plot[column_name].drop_duplicates()))

        for num, type in enumerate(types):
            values = list(df_plot.loc[df_plot["tag"] == type, "percent"])
            df_plot[df_plot['tag'] == type].plot.bar(
                x=column_name, y='percent', ax=ax, stacked=True,
                bottom=margin_bottom, label=type, color=colors[num * 3 + 1])
            margin_bottom += values
        ax.legend(loc='center left', bbox_to_anchor=(1, 0.5))
        plt.title("tag fraction")
        fig.savefig(plot_file)

    @log
    def run(self):
        stats = pd.Series()
        outdir = self.outdir
        sample = self.sample
        UMI_tag_file = f'{outdir}/{sample}_umi_tag.tsv'
        tsne_tag_file = f'{outdir}/{sample}_tsne_tag.tsv'
        cluster_count_file = f'{outdir}/{sample}_cluster_count.tsv'
        cluster_plot = f'{outdir}/{sample}_cluster_plot.pdf'
        if self.combine_cluster:
            combine_cluster_count_file = f'{outdir}/{sample}_combine_cluster_count.tsv'
            combine_cluster_plot = f'{outdir}/{sample}_combine_cluster_plot.pdf'

        mapped_read = self.df_read_count['read_count'].sum()

        # in cell
        df_read_count_in_cell = self.df_read_count[self.df_read_count.index.isin(self.match_barcode)]
        mapped_read_in_cell = int(df_read_count_in_cell['read_count'].sum())
        stats = stats.append(pd.Series(
            format_stat(mapped_read_in_cell, mapped_read),
            index=['Mapped Reads in Cells']
        ))

        # UMI
        tag_name = df_read_count_in_cell.columns[0]
        df_UMI_in_cell = df_read_count_in_cell.reset_index().groupby([
            'barcode', tag_name]).agg({'UMI': 'count'})
        df_UMI_in_cell = df_UMI_in_cell.reset_index()
        df_UMI_in_cell = df_UMI_in_cell.pivot(
            index='barcode', columns=tag_name, values='UMI')
        df_cell = pd.DataFrame(index=self.match_barcode)
        df_UMI_cell = pd.merge(
            df_cell,
            df_UMI_in_cell,
            how="left",
            left_index=True,
            right_index=True)

        # fillna
        df_UMI_cell.fillna(0, inplace=True)
        df_UMI_cell = df_UMI_cell.astype(int)

        # UMI
        UMIs = df_UMI_cell.apply(sum, axis=1)
        median = round(np.median(UMIs), 2)
        mean = round(np.mean(UMIs), 2)
        stats = stats.append(pd.Series(
            str(median),
            index=['Median UMI per Cell']
        ))

        stats = stats.append(pd.Series(
            str(mean),
            index=['Mean UMI per Cell']
        ))

        UMI_min = Count_tag.get_UMI_min(df_UMI_cell, self.UMI_min)
        Count_tag.run.logger.info(f'UMI_min: {UMI_min}')
        SNR_min = self.get_SNR_min(df_UMI_cell, self.dim, self.SNR_min, UMI_min)
        Count_tag.run.logger.info(f'SNR_min: {SNR_min}')
        df_UMI_cell["tag"] = df_UMI_cell.apply(
            Count_tag.tag_type, UMI_min=UMI_min, SNR_min=SNR_min, dim=self.dim, no_noise=self.no_noise, axis=1)
        df_UMI_cell.to_csv(UMI_tag_file, sep="\t")

        df_tsne = pd.read_csv(self.tsne_file, sep="\t", index_col=0)
        df_tsne_tag = pd.merge(
            df_tsne,
            df_UMI_cell,
            how="left",
            left_index=True,
            right_index=True)

        if self.combine_cluster:
            df_combine_cluster = pd.read_csv(
                self.combine_cluster, sep="\t", header=None)
            df_combine_cluster.columns = ["cluster", "combine_cluster"]
            df_tsne_combine_cluster_tag = pd.merge(
                df_tsne_tag, df_combine_cluster,
                on=["cluster"], how="left", left_index=True).set_index(df_tsne_tag.index)
            df_tsne_combine_cluster_tag.to_csv(tsne_tag_file, sep="\t")
        else:
            df_tsne_tag.to_csv(tsne_tag_file, sep="\t")

        self.write_and_plot(
            df=df_tsne_tag, column_name="cluster", count_file=cluster_count_file,
            plot_file=cluster_plot
        )

        if self.combine_cluster:
            self.write_and_plot(
                df=df_tsne_combine_cluster_tag,
                column_name="combine_cluster",
                count_file=combine_cluster_count_file,
                plot_file=combine_cluster_plot
            )

        df_tag_count = df_UMI_cell["tag"].value_counts().reset_index()
        df_tag_count.columns = ["item", "count"]
        for index, row in df_tag_count.iterrows():
            stats = stats.append(pd.Series(
                format_stat(row['count'], self.cell_total),
                index=[row['item'] + ' Cells']
        ))
        self.stats = stats

    def report(self):

        self.stat_file = f'{self.outdir}/stat.txt'
        self.stats.to_csv(self.stat_file, sep=':', header=False)
        t = reporter(
        name='count_tag', 
        assay=self.assay, 
        sample=self.sample,
        stat_file=self.stat_file, 
        outdir=self.outdir + '/..')
        t.get_report()



import argparse
from .Count_tag import Count_tag

def get_opts_count_tag(parser, sub_program):
    parser.add_argument(
        "--UMI_min",
        help="cells have tag_UMI>=UMI_min are considered as valid cell",
        default="auto")
    parser.add_argument("--dim", help="tag dimension", default=1)
    parser.add_argument(
        "--SNR_min",
        help="minimum signal to noise ratio",
        default="auto")
    parser.add_argument(
        "--combine_cluster",
        help="conbine cluster tsv file",
        default=None)
    parser.add_argument("--match_dir", help="matched scRNA-Seq CeleScope directory path", required=True)
    if sub_program:
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument('--assay', help='assay', required=True)
        parser.add_argument("--read_count_file", help="tag read count file")


def count_tag(args):

    count_tag_object = Count_tag(
        args.sample,
        args.outdir,
        args.assay,
        args.read_count_file,
        args.match_dir,
        args.UMI_min,
        args.SNR_min,
        args.combine_cluster,
        args.dim,
    )
    count_tag_object.run()
    count_tag_object.report()


import os
import glob
import sys
import argparse
import re
from collections import defaultdict
from celescope.__init__ import __CONDA__
from celescope.tools.utils import merge_report
from celescope.tools.utils import parse_map_col4, link_data
from celescope.tools.__init__ import __PATTERN_DICT__

class Multi():

    def __init__(self, __ASSAY__, __STEPS__, __CONDA__):
        self.__ASSAY__ = __ASSAY__
        self.__STEPS__ = __STEPS__
        self.__CONDA__ = __CONDA__
        self.__APP__ = 'celescope'
        self.col4_default = 'auto'
        self.last_step = ''

    def multi_opts(self):
        readme = f'{self.__ASSAY__} multi-samples'
        parser = argparse.ArgumentParser(readme)
        parser.add_argument('--mod', help='mod, sjm or shell', choices=['sjm', 'shell'], default='sjm')
        parser.add_argument(
            '--mapfile',
            help='''
                tsv file, 4 columns:
                1st col: LibName;
                2nd col: DataDir;
                3rd col: SampleName;
                4th col: Cell number or match_dir, optional;
            ''',
            required=True)
        parser.add_argument('--chemistry', choices=__PATTERN_DICT__.keys(), help='chemistry version', default='auto')
        parser.add_argument('--whitelist', help='cellbarcode list')
        parser.add_argument('--linker', help='linker')
        parser.add_argument('--pattern', help='read1 pattern')
        parser.add_argument('--outdir', help='output dir', default="./")
        parser.add_argument(
            '--adapt',
            action='append',
            help='adapter sequence',
            default=[
                'polyT=A{15}',
                'p5=AGATCGGAAGAGCACACGTCTGAACTCCAGTCAC'])
        parser.add_argument(
            '--minimum_length',
            dest='minimum_length',
            help='minimum_length',
            default=20)
        parser.add_argument(
            '--nextseq-trim',
            dest='nextseq_trim',
            help='nextseq_trim',
            default=20)
        parser.add_argument('--overlap', help='minimum overlap length', default=10)
        parser.add_argument('--lowQual', type=int, help='max phred of base as lowQual', default=0)
        parser.add_argument(
            '--lowNum',
            type=int,
            help='max number with lowQual allowed',
            default=2)
        parser.add_argument(
            '--rm_files',
            action='store_true',
            help='remove redundant fq.gz and bam after running')
        parser.add_argument('--steps_run', help='steps to run', default='all')
        parser.add_argument('--debug', help='debug or not', action='store_true')
        self.parser = parser
        return parser

    def STAR_args(self):
        self.parser.add_argument('--starMem', help='starMem', default=30)
        self.parser.add_argument('--genomeDir', help='genome index dir', required=True)
        self.parser.add_argument(
            '--gtf_type',
            help='Specify attribute type in GTF annotation, default=exon',
            default='exon')
        self.parser.add_argument('--thread', help='thread', default=6)
        self.parser.add_argument('--out_unmapped', help='out_unmapped', action='store_true')
        self.parser.add_argument('--outFilterMatchNmin', help='STAR outFilterMatchNmin', default=0)

    def analysis_args(self):
        self.parser.add_argument('--save_rds', action='store_true', help='write rds to disk')
        self.parser.add_argument('--type_marker_tsv', help='cell type marker tsv')

    def custome_args(self):
        self.STAR_args()
        self.analysis_args()

    def parse_args(self):
        self.multi_opts()
        self.custome_args()
        self.args = self.parser.parse_args()
        # read args
        self.outdir = self.args.outdir
        self.chemistry = self.args.chemistry
        self.pattern = self.args.pattern
        self.whitelist = self.args.whitelist
        self.linker = self.args.linker
        self.lowQual = self.args.lowQual
        self.lowNum = self.args.lowNum
        self.overlap = self.args.overlap
        self.minimum_length = self.args.minimum_length
        self.mod = self.args.mod
        self.rm_files = self.args.rm_files
        self.steps_run = self.args.steps_run
        if self.__CONDA__ == 'celescope_RD':
            self.debug_str = '--debug'
        else:
            self.debug_str = Multi.arg_str(self.args.debug, 'debug')

    @staticmethod
    def arg_str(arg, arg_name):
        '''
        return action store_true arguments as string
        '''
        if arg:
            return '--' + arg_name
        return ''

    def read_STAR_args(self):
        self.thread = self.args.thread
        self.genomeDir = self.args.genomeDir
        self.starMem = self.args.starMem
        self.gtf_type = self.args.gtf_type
        self.out_unmapped = Multi.arg_str(self.args.out_unmapped, 'out_unmapped')
        self.outFilterMatchNmin = self.args.outFilterMatchNmin
    
    def read_analysis_args(self):
        self.save_rds = self.args.save_rds
        self.save_rds_str = Multi.arg_str(self.save_rds, 'save_rds')
        self.type_marker_tsv = self.args.type_marker_tsv

    def read_custome_args(self):
        self.read_STAR_args()
        self.read_analysis_args()

    def prepare(self):
        # parse_mapfile
        self.fq_dict, self.col4_dict = parse_map_col4(self.args.mapfile, self.col4_default)

        # link
        link_data(self.outdir, self.fq_dict)      

        # mk log dir
        self.logdir = self.outdir + '/log'
        os.system('mkdir -p %s' % (self.logdir))

        # script init
        self.sjm_cmd = 'log_dir %s\n' % (self.logdir)
        self.sjm_order = ''
        self.shell_dict = defaultdict(str)

        # outdir dict
        self.outdir_dic = {}
        for sample in self.fq_dict:
            self.outdir_dic[sample] = {}
            index = 0
            for step in self.__STEPS__:
                step_outdir = f"{self.outdir}/{sample}/{index:02d}.{step}"
                self.outdir_dic[sample].update({step: step_outdir})
                index += 1
    
    def generate_cmd(self, cmd, step, sample, m, x):
        self.sjm_cmd += f'''
job_begin
    name {step}_{sample}
    sched_options -w n -cwd -V -l vf={m}g,p={x}
    cmd source activate {self.__CONDA__}; {cmd}
job_end
'''

    def process_cmd(self, cmd, step, sample, m=1, x=1):
        self.generate_cmd(cmd, step, sample, m=m, x=x)
        self.shell_dict[sample] += cmd + '\n'
        if self.last_step:
            self.sjm_order += f'order {step}_{sample} after {self.last_step}_{sample}\n'
        self.last_step = step

    def generate_first(self, cmd, step, sample, m=1, x=1):
        self.generate_cmd(cmd, step, sample, m=1, x=1)
        self.shell_dict[sample] += cmd + '\n'
        self.last_step = step

    def generate_other(self, cmd, step, sample, m=1, x=1):
        self.generate_cmd(cmd, step, sample, m=1, x=1)
        self.shell_dict[sample] += cmd + '\n'
        self.sjm_order += f'order {step}_{sample} after {self.last_step}_{sample}\n'
        self.last_step = step

    def sample(self, sample):
        step = "sample"
        arr = self.fq_dict[sample]
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--chemistry {self.chemistry} '
            f'--fq1 {arr[0]}'
        )
        self.process_cmd(cmd, step, sample, m=1, x=1)
    
    def barcode(self, sample):
        # barcode
        arr = self.fq_dict[sample]
        step = "barcode"
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--chemistry {self.chemistry} '
            f'--fq1 {arr[0]} --fq2 {arr[1]} '
            f'--pattern {self.pattern} --whitelist {self.whitelist} --linker {self.linker} '
            f'--lowQual {self.lowQual} --thread {self.thread} '
            f'--lowNum {self.lowNum} '

        )
        self.process_cmd(cmd, step, sample, m=5, x=1)

    def cutadapt(self, sample):
        # adapt
        step = "cutadapt"
        fq = f'{self.outdir_dic[sample]["barcode"]}/{sample}_2.fq.gz'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step } '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--fq {fq} '
            f'--overlap {self.overlap} '
            f'--minimum_length {self.minimum_length} '
        )
        self.process_cmd(cmd, step, sample, m=5, x=1)

    def STAR(self, sample):
        step = 'STAR'
        fq = f'{self.outdir_dic[sample]["cutadapt"]}/{sample}_clean_2.fq.gz'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--fq {fq} '
            f'--genomeDir {self.genomeDir} '
            f'--thread {self.thread} '
            f'{self.debug_str} '
            f'--outFilterMatchNmin {self.outFilterMatchNmin} '
            f'{self.out_unmapped} '
        )
        self.process_cmd(cmd, step, sample, m=self.starMem, x=self.thread)

    def featureCounts(self, sample):
        step = 'featureCounts'
        input = f'{self.outdir_dic[sample]["STAR"]}/{sample}_Aligned.sortedByCoord.out.bam'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--input {input} --gtf_type {self.gtf_type} '
            f'--genomeDir {self.genomeDir} '
            f'--thread {self.thread} '
            f'--gtf_type {self.gtf_type} '
        )
        self.process_cmd(cmd, step, sample, m=5, x=self.thread)

    def count(self, sample):
        step = 'count'
        bam = f'{self.outdir_dic[sample]["featureCounts"]}/{sample}_name_sorted.bam'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--bam {bam} '
            f'--cells {self.col4_dict[sample]} '
            f'--genomeDir {self.genomeDir} '
        )
        self.process_cmd(cmd, step, sample, m=10, x=1)

    def analysis(self, sample):
        step = 'analysis'
        matrix_file = f'{self.outdir_dic[sample]["count"]}/{sample}_matrix.tsv.gz'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step } '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--matrix_file {matrix_file} '
            f'{self.save_rds_str} '
            f'--type_marker_tsv {self.type_marker_tsv} '
        )
        self.process_cmd(cmd, step, sample, m=10, x=1)

    def run_steps(self):
        if self.steps_run == 'all':
            self.steps_run = self.__STEPS__
        elif self.steps_run:
            self.steps_run = self.steps_run.strip().split(',')

        for sample in self.fq_dict:
            self.last_step = ''
            for step in self.steps_run:
                eval(f'self.{step}(sample)')

    def end(self):
        if self.mod == 'sjm':
            step = 'merge_report'
            merge_report(
                self.fq_dict,
                self.__STEPS__,
                self.last_step,
                self.sjm_cmd,
                self.sjm_order,
                self.logdir,
                self.__CONDA__,
                self.outdir,
                self.rm_files,
            )
        if self.mod == 'shell':
            os.system('mkdir -p ./shell/')
            for sample in self.shell_dict:
                with open(f'./shell/{sample}.sh', 'w') as f:
                    f.write(self.shell_dict[sample])

    def run(self):
        self.multi_opts()
        self.custome_args()
        self.parse_args()
        self.read_custome_args()
        self.prepare()
        self.run_steps()
        self.end()



    


import os
import sys
import json
import numpy as np
import pandas as pd
import glob
from celescope.tools.report import reporter
from celescope.tools.utils import glob_genomeDir, log, parse_annovar
from celescope.tools.utils import cluster_tsne_list, parse_match_dir

class Analysis():

    def __init__(
        self,
        sample,
        outdir,
        assay,
        match_dir,
        step,     
    ):
        self.sample = sample
        self.outdir = outdir
        self.assay = assay
        self.match_dir = match_dir
        self.step = step

        match_dict = parse_match_dir(match_dir)
        tsne_df_file = match_dict['tsne_coord']
        marker_df_file = match_dict['markers']
        self.tsne_df = pd.read_csv(tsne_df_file, sep="\t")
        self.marker_df = pd.read_csv(marker_df_file, sep="\t")
        self.tsne_df.rename(columns={"Unnamed: 0": "barcode"}, inplace=True)
        self.cluster_tsne = cluster_tsne_list(self.tsne_df)

        if not os.path.exists(outdir):
            os.system('mkdir -p %s' % outdir)

    def add_attrs(self, *kwargs):
        for kwarg in kwargs:
            setattr(self, kwarg, kwargs[kwarg])

    @staticmethod
    def get_table(title, id, df_table):
        """
        return html code
        """
        table_dict = {}
        table_dict['title'] = title
        table_dict['table'] = df_table.to_html(
            escape=False,
            index=False,
            table_id=id,
            justify="center")
        return table_dict

    def report(self, stat=True):
        if stat:
            stat_file = self.outdir + "/stat.txt"
        else:
            stat_file = ''
        t = reporter(
        name=self.step,
        assay=self.assay,
        sample=self.sample,
        outdir=self.outdir + '/..',
        stat_file=stat_file)
        t.get_report()

    def get_cluster_tsne(self, colname, show_tag=True, dfname='tsne_df'):
        """
        tSNE_1	tSNE_2	cluster Gene_Counts
        return data list
        """
        tsne_df = getattr(self, dfname)
        sum_df = tsne_df.groupby([colname]).agg("count").iloc[:, 0]
        percent_df = sum_df.transform(lambda x: round(x / sum(x) * 100, 2))
        res = []
        for cluster in sorted(tsne_df[colname].unique()):
            sub_df = tsne_df[tsne_df[colname] == cluster]
            if show_tag:
                name = f"{colname} {cluster}({percent_df[cluster]}%)"
            else:
                name = f"{cluster}({percent_df[cluster]}%)"
            tSNE_1 = list(sub_df.tSNE_1)
            tSNE_2 = list(sub_df.tSNE_2)
            res.append({"name": name, "tSNE_1": tSNE_1, "tSNE_2": tSNE_2})
        return res

    def get_gene_tsne(self):
        """
        return data dic
        """
        tsne_df = self.tsne_df
        tSNE_1 = list(tsne_df.tSNE_1)
        tSNE_2 = list(tsne_df.tSNE_2)
        Gene_Counts = list(tsne_df.Gene_Counts)
        res = {"tSNE_1": tSNE_1, "tSNE_2": tSNE_2, "Gene_Counts": Gene_Counts}
        return res

    def process_marker_table(self):
        """
        return html code
        """
        marker_df = self.marker_df.loc[:, ["cluster", "gene",
                                    "avg_logFC", "pct.1", "pct.2", "p_val_adj"]]
        marker_df["cluster"] = marker_df["cluster"].apply(lambda x: f"cluster {x}")
        return marker_df

    def report_prepare(self, **kwargs):
        json_file = self.outdir + '/../.data.json'
        if not os.path.exists(json_file):
            data = {}
        else:
            fh = open(json_file)
            data = json.load(fh)
            fh.close()

        for key in kwargs:
            data[key] = kwargs[key]

        with open(json_file, 'w') as fh:
            json.dump(data, fh)

    def get_marker_gene_table(self):
        marker_df = self.process_marker_table()
        table_dict = Analysis.get_table(
            title='Marker Genes by Cluster',
            id='marker_gene_table',
            df_table=marker_df,
        )
        return table_dict

    def marker_table(self):
        """
        return html code
        """
        marker_df = self.marker_df.loc[:, ["cluster", "gene",
                                    "avg_logFC", "pct.1", "pct.2", "p_val_adj"]]
        marker_df["cluster"] = marker_df["cluster"].apply(lambda x: f"cluster {x}")
        marker_gene_table = marker_df.to_html(
            escape=False,
            index=False,
            table_id="marker_gene_table",
            justify="center")
        return marker_gene_table

    def run(self):
        cluster_tsne = self.get_cluster_tsne(colname='cluster')
        gene_tsne = self.get_gene_tsne()
        table_dict = self.get_marker_gene_table()
        self.report_prepare(
            cluster_tsne=cluster_tsne,
            gene_tsne=gene_tsne,
            table_dict=table_dict,
        )
        self.report()


#!/bin/env python
# coding=utf8

import os
import sys
import json
import logging
import re
import numpy as np
import pandas as pd
import glob
from scipy.io import mmwrite
from scipy.sparse import csr_matrix
from celescope.tools.report import reporter
from celescope.tools.utils import glob_genomeDir, log, gene_convert
from celescope.rna.__init__ import __ASSAY__
from celescope.tools.Analysis import Analysis

toolsdir = os.path.dirname(__file__)


def report_prepare(outdir, tsne_df, marker_df):
    json_file = outdir + '/../.data.json'
    if not os.path.exists(json_file):
        data = {}
    else:
        fh = open(json_file)
        data = json.load(fh)
        fh.close()

    data["cluster_tsne"] = cluster_tsne_list(tsne_df)
    data["gene_tsne"] = gene_tsne_list(tsne_df)
    data["marker_gene_table"] = marker_table(marker_df)

    with open(json_file, 'w') as fh:
        json.dump(data, fh)


def cluster_tsne_list(tsne_df):
    """
    tSNE_1	tSNE_2	cluster Gene_Counts
    return data list
    """
    sum_df = tsne_df.groupby(["cluster"]).agg("count").iloc[:, 0]
    percent_df = sum_df.transform(lambda x: round(x / sum(x) * 100, 2))
    res = []
    for cluster in sorted(tsne_df.cluster.unique()):
        sub_df = tsne_df[tsne_df.cluster == cluster]
        name = "cluster {cluster}({percent}%)".format(
            cluster=cluster, percent=percent_df[cluster])
        tSNE_1 = list(sub_df.tSNE_1)
        tSNE_2 = list(sub_df.tSNE_2)
        res.append({"name": name, "tSNE_1": tSNE_1, "tSNE_2": tSNE_2})
    return res


def gene_tsne_list(tsne_df):
    """
    return data dic
    """
    tSNE_1 = list(tsne_df.tSNE_1)
    tSNE_2 = list(tsne_df.tSNE_2)
    Gene_Counts = list(tsne_df.Gene_Counts)
    res = {"tSNE_1": tSNE_1, "tSNE_2": tSNE_2, "Gene_Counts": Gene_Counts}
    return res


def marker_table(marker_df):
    """
    return html code
    """
    marker_df = marker_df.loc[:, ["cluster", "gene",
                                  "avg_logFC", "pct.1", "pct.2", "p_val_adj"]]
    marker_df["cluster"] = marker_df["cluster"].apply(lambda x: f"cluster {x}")
    marker_gene_table = marker_df.to_html(
        escape=False,
        index=False,
        table_id="marker_gene_table",
        justify="center")
    return marker_gene_table


@log
def generate_matrix(gtf_file, matrix_file):

    id_name = gene_convert(gtf_file)
    matrix = pd.read_csv(matrix_file, sep="\t")

    gene_name_col = matrix.geneID.apply(lambda x: id_name[x])
    matrix.geneID = gene_name_col
    matrix = matrix.drop_duplicates(subset=["geneID"], keep="first")
    matrix = matrix.dropna()
    matrix = matrix.rename({"geneID": ""}, axis='columns')
    return matrix


@log
def seurat(sample, outdir, matrix_file, save_rds):
    app = toolsdir + "/run_analysis.R"
    cmd = (
        f'Rscript {app} --sample {sample} --outdir {outdir} --matrix_file {matrix_file} '
        f'--save_rds {save_rds}'
    )
    seurat.logger.info(cmd)
    os.system(cmd)


@log
def auto_assign(sample, outdir, type_marker_tsv):
    rds = f'{outdir}/{sample}.rds'
    app = toolsdir + "/auto_assign.R"
    cmd = (
        f'Rscript {app} '
        f'--rds {rds} '
        f'--type_marker_tsv {type_marker_tsv} '
        f'--outdir {outdir} '
        f'--sample {sample} '
    )
    auto_assign.logger.info(cmd)
    os.system(cmd)


@log
def analysis(args):

    # check dir
    outdir = args.outdir
    sample = args.sample
    assay = args.assay 

    matrix_file = args.matrix_file
    save_rds = args.save_rds
    type_marker_tsv = args.type_marker_tsv
    auto_assign_bool = False
    if type_marker_tsv and type_marker_tsv != 'None':
        auto_assign_bool = True
    if auto_assign_bool:
        save_rds = True

    if not os.path.exists(outdir):
        os.system('mkdir -p %s' % (outdir))

    # run_R
    seurat(sample, outdir, matrix_file, save_rds)

    # auto_assign
    if auto_assign_bool:
        auto_assign(sample, outdir, type_marker_tsv)

    # report
    ana = Analysis(     
        sample,
        outdir,
        assay,
        match_dir=outdir+'/../',
        step='analysis',     
    )
    ana.run()


def get_opts_analysis(parser, sub_program):
    if sub_program:
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument('--matrix_file', help='matrix file', required=True)
        parser.add_argument('--assay', help='assay', required=True)
    parser.add_argument('--save_rds', action='store_true', help='write rds to disk')
    parser.add_argument('--type_marker_tsv', help='cell type marker tsv')




#!/bin/env python
# coding=utf8

import os
import re
import sys
import json
import logging
import subprocess
import glob
import pandas as pd
from celescope.tools.utils import format_number, log, format_stat
from celescope.tools.utils import glob_genomeDir
from celescope.tools.report import reporter

class Step_mapping():

    def __init__(self, sample, outdir, assay, thread, fq, genomeDir, out_unmapped=False, debug=False, outFilterMatchNmin=0):
        self.sample = sample
        self.outdir = outdir
        self.assay = assay
        self.thread = thread
        self.fq = fq
        self.genomeDir = genomeDir
        self.out_unmapped = out_unmapped
        self.debug = debug
        self.outFilterMatchNmin = outFilterMatchNmin
        if not os.path.exists(outdir):
            os.system('mkdir -p %s' % (outdir))
        self.stats = pd.Series()
        self.stats_file = f'{outdir}/stat.txt'
        self.step_name = 'STAR'

    def format_stat(self):
        fh1 = open(self.STAR_map_log, 'r')
        UNIQUE_READS = []
        MULTI_MAPPING_READS = []
        for line in fh1:
            if line.strip() == '':
                continue
            if re.search(r'Uniquely mapped reads', line):
                UNIQUE_READS.append(line.strip().split()[-1])
            if re.search(r'of reads mapped to too many loci', line):
                MULTI_MAPPING_READS.append(line.strip().split()[-1])
        fh1.close()

        fh2 = open(self.picard_region_log, 'r')
        region_dict = {}
        while True:
            line = fh2.readline()
            if not line:
                break
            if line.startswith('## METRICS CLASS'):
                header = fh2.readline().strip().split('\t')
                data = fh2.readline().strip().split('\t')
                region_dict = dict(zip(header, data))
                break
        fh2.close()
        
        Total = float(region_dict['PF_ALIGNED_BASES'])
        Exonic_Regions = int(region_dict['UTR_BASES']) + \
            int(region_dict['CODING_BASES'])
        Intronic_Regions = int(region_dict['INTRONIC_BASES'])
        Intergenic_Regions = int(region_dict['INTERGENIC_BASES'])

        region_dict['Exonic_Regions'] = "{}({:.2%})".format(
            format_number(Exonic_Regions), Exonic_Regions / Total)
        region_dict['Intronic_Regions'] = "{}({:.2%})".format(
            format_number(Intronic_Regions), Intronic_Regions / Total)
        region_dict['Intergenic_Regions'] = "{}({:.2%})".format(
            format_number(Intergenic_Regions), Intergenic_Regions / Total)

        self.stats = self.stats.append(pd.Series(
            f'{format_number(int(UNIQUE_READS[0]))}({UNIQUE_READS[1]})',
            index=['Uniquely Mapped Reads']
        ))
        self.stats = self.stats.append(pd.Series(
            f'{format_number(int(MULTI_MAPPING_READS[0]))}({MULTI_MAPPING_READS[1]})',
            index=['Multi-Mapped Reads']
        ))
        # ribo
        if self.debug:
            f = open(self.ribo_log, 'r')
            for line in f:
                if line.find('#Matched') != -1:
                    items = line.split()
                    Reads_Mapped_to_rRNA = int(items[1])
                if line.find('#Total') != -1:
                    items = line.split()
                    Reads_Total = int(items[1])

            self.stats = self.stats.append(pd.Series(
                format_stat(Reads_Mapped_to_rRNA, Reads_Total),
                index=['Reads Mapped to rRNA']
            ))
            f.close()

        self.stats = self.stats.append(pd.Series(
            region_dict['Exonic_Regions'],
            index=['Base Pairs Mapped to Exonic Regions']
        ))
        self.stats = self.stats.append(pd.Series(
            region_dict['Intronic_Regions'],
            index=['Base Pairs Mapped to Intronic Regions']
        ))
        self.stats = self.stats.append(pd.Series(
            region_dict['Intergenic_Regions'],
            index=['Base Pairs Mapped to Intergenic Regions']
        ))
        self.plot = {'region_labels': ['Exonic Regions', 'Intronic Regions', 'Intergenic Regions'],
                'region_values': [Exonic_Regions, Intronic_Regions, Intergenic_Regions]}   


        self.stats.to_csv(self.stats_file, sep=':', header=False)

    @log
    def ribo(self):
        import celescope
        root_path = os.path.dirname(celescope.__file__)
        human_ribo_fa = f'{root_path}/data/rRNA/human_ribo.fasta'
        self.ribo_log = f'{self.outdir}/{self.sample}_ribo_log.txt'
        self.ribo_run_log = f'{self.outdir}/{self.sample}_ribo_run.log'
        cmd = (
            f'bbduk.sh '
            f'in1={self.fq} '
            f'ref={human_ribo_fa} '
            f'stats={self.ribo_log} '
            f'overwrite=t '
            f'> {self.ribo_run_log} 2>&1 '
        )
        Step_mapping.ribo.logger.info(cmd)
        subprocess.check_call(cmd, shell=True)
    
    @log
    def STAR(self):
        self.outPrefix = f'{self.outdir}/{self.sample}_'
        self.STAR_bam = f'{self.outdir}/{self.sample}_Aligned.sortedByCoord.out.bam'
        self.STAR_map_log = f'{self.outdir}/{self.sample}_Log.final.out'    
        cmd = ['STAR', '--runThreadN', str(self.thread), '--genomeDir', self.genomeDir,
            '--readFilesIn', self.fq, '--readFilesCommand', 'zcat', '--outFilterMultimapNmax',
            '1', '--outFileNamePrefix', self.outPrefix, '--outSAMtype', 'BAM', 'SortedByCoordinate',
            '--outFilterMatchNmin', str(self.outFilterMatchNmin)]
        if self.out_unmapped:
            cmd += ['--outReadsUnmapped', 'Fastx']
        Step_mapping.STAR.logger.info(' '.join(cmd))
        subprocess.check_call(cmd)

    @log
    def picard(self):
        self.refFlat, self.gtf = glob_genomeDir(self.genomeDir)
        self.picard_region_log = f'{self.outdir}/{self.sample}_region.log'
        cmd = [
            'picard',
            '-Xmx20G',
            '-XX:ParallelGCThreads=4',
            'CollectRnaSeqMetrics',
            'I=%s' %
            (self.STAR_bam),
            'O=%s' %
            (self.picard_region_log),
            'REF_FLAT=%s' %
            (self.refFlat),
            'STRAND=NONE',
            'VALIDATION_STRINGENCY=SILENT']
        res = subprocess.run(cmd, stderr=subprocess.STDOUT, stdout=subprocess.PIPE)

    @log
    def run(self):
        self.STAR()
        self.picard()
        if self.debug:
            self.ribo()
        self.format_stat()
        self.report()
    
    def report(self):
        t = reporter(
            name=self.step_name,
            assay=self.assay,
            sample=self.sample,
            stat_file=self.stats_file,
            outdir=self.outdir + '/..',
            plot=self.plot)
        t.get_report()


def STAR(args):
    mapping = Step_mapping(
        args.sample, 
        args.outdir, 
        args.assay, 
        args.thread,
        args.fq, 
        args.genomeDir, 
        args.out_unmapped, 
        args.debug,
        args.outFilterMatchNmin)
    mapping.run()


def get_opts_STAR(parser, sub_program):
    if sub_program:
        parser.add_argument('--fq', required=True)
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument('--thread', default=1)
        parser.add_argument('--assay', help='assay', required=True)
        parser.add_argument('--debug', help='debug', action='store_true')
    parser.add_argument('--outFilterMatchNmin', help='STAR outFilterMatchNmin', default=0)
    parser.add_argument('--out_unmapped', help='out_unmapped', action='store_true')
    parser.add_argument('--genomeDir', help='genome directory', required=True)



__VERSION__ = "1.1.6"
__CONDA__ = "celescope_RD"

ASSAY_DICT = {
    "rna": "Single Cell RNA-Seq",
    "rna_virus": "Single Cell RNA-Seq Virus",
    'capture_virus': "Single Cell Capture Virus",
    'fusion': "Single Cell Fusion Gene",
    'smk': "Single Cell SMK",
    'vdj': "Single Cell VDJ",
    'mut': 'Single Cell Insertion and Deletion',
    'hla': 'Single Cell HLA',
    'capture_rna': 'Single Cell Capture RNA',
    'snp': 'Single Cell Variant',
    'tag': 'Single Cell tag',
    'citeseq': 'Single Cell CITE-Seq',
    'tcr_fl': 'Single Cell full length TCR',
}



import os
import sys
import json
import numpy as np
import pandas as pd
import glob
from celescope.tools.report import reporter
from celescope.tools.utils import log
from celescope.tools.Analysis import Analysis

class Analysis_tag(Analysis):

    def run(self, tsne_tag_file):
        cluster_tsne = self.get_cluster_tsne(colname='cluster')
        self.tsne_tag_df = pd.read_csv(tsne_tag_file, sep="\t", index_col=0)
        feature_tsne = self.get_cluster_tsne(colname='tag', show_tag=False, dfname='tsne_tag_df')
        marker_gene_table = self.marker_table()
        self.report_prepare(
            cluster_tsne=cluster_tsne, 
            feature_tsne=feature_tsne,
            marker_gene_table=marker_gene_table,
        )
        self.report(stat=False)








import argparse
from .Analysis_tag import Analysis_tag

def get_opts_analysis_tag(parser, sub_program):
    if sub_program:
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument('--match_dir', help='match_dir', required=True)
        parser.add_argument('--tsne_tag_file', help='tsne tag file', required=True)
        parser.add_argument('--assay', help='assay', required=True)


def analysis_tag(args):

    analysis_tag_object = Analysis_tag(
        args.sample,
        args.outdir,
        args.assay,
        args.match_dir,
        'analysis_tag',
    )
    analysis_tag_object.run(args.tsne_tag_file)


from celescope.__init__ import __CONDA__
from celescope.fusion.__init__ import __STEPS__, __ASSAY__
from celescope.tools.Multi import Multi


class Multi_fusion(Multi):
    def custome_args(self):
        self.STAR_args()
        self.parser.add_argument("--fusion_pos",
        help="first base position of the second gene(0-start),tsv file", required=True)
        self.parser.add_argument("--UMI_min", default=1)

    def read_custome_args(self):
        self.read_STAR_args()
        self.fusion_pos = self.args.fusion_pos
        self.UMI_min = self.args.UMI_min

    def STAR_fusion(self, sample):
        step = 'STAR_fusion'
        input_read = f'{self.outdir_dic[sample]["cutadapt"]}/{sample}_clean_2.fq.gz'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--input_read {input_read} '
            f'--genomeDir {self.genomeDir} '
            f'--thread {self.thread} '
        )
        self.process_cmd(cmd, step, sample, m=self.starMem, x=self.thread)

    def count_fusion(self, sample):
        step = 'count_fusion'
        bam = f'{self.outdir_dic[sample]["STAR_fusion"]}/{sample}_Aligned.sortedByCoord.out.bam'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--bam {bam} '
            f'--UMI_min {self.UMI_min} '
            f'--match_dir {self.col4_dict[sample]} '
            f'--fusion_pos {self.fusion_pos} '
        )
        self.process_cmd(cmd, step, sample, m=15, x=1)


def main():
    multi = Multi_fusion(__ASSAY__, __STEPS__, __CONDA__)
    multi.col4_default = None
    multi.run()


if __name__ == '__main__':
    main()





#!/bin/env python
# coding=utf8

import os
import sys
import json
import logging
import re
import numpy as np
import pandas as pd
import glob
from scipy.io import mmwrite
from scipy.sparse import csr_matrix
from celescope.tools.report import reporter
from celescope.tools.utils import glob_genomeDir

logger1 = logging.getLogger(__name__)


def report_prepare(outdir, tsne_df, marker_df, feature_df):
    json_file = outdir + '/../.data.json'
    if not os.path.exists(json_file):
        data = {}
    else:
        fh = open(json_file)
        data = json.load(fh)
        fh.close()

    data["cluster_tsne"] = cluster_tsne_list(tsne_df, "cluster")
    data["cluster_tsne_1"] = cluster_tsne_list(
        feature_df, "tag", show_tag=False)
    data["marker_gene_table"] = marker_table(marker_df)

    with open(json_file, 'w') as fh:
        json.dump(data, fh)


def cluster_tsne_list(tsne_df, colname, show_tag=True):
    """
    tSNE_1	tSNE_2	cluster Gene_Counts
    return data list
    """
    sum_df = tsne_df.groupby([colname]).agg("count").iloc[:, 0]
    percent_df = sum_df.transform(lambda x: round(x / sum(x) * 100, 2))
    res = []
    for cluster in sorted(tsne_df[colname].unique()):
        sub_df = tsne_df[tsne_df[colname] == cluster]
        if show_tag:
            name = f"{colname} {cluster}({percent_df[cluster]}%)"
        else:
            name = f"{cluster}({percent_df[cluster]}%)"
        tSNE_1 = list(sub_df.tSNE_1)
        tSNE_2 = list(sub_df.tSNE_2)
        res.append({"name": name, "tSNE_1": tSNE_1, "tSNE_2": tSNE_2})
    return res


def feature_tsne_list(feature_df):
    """
    return data dic
    """
    colname = "tag"
    tSNE_1 = list(feature_df.tSNE_1)
    tSNE_2 = list(feature_df.tSNE_2)
    feature_val = list(feature_df[colname])
    res = {"tSNE_1": tSNE_1, "tSNE_2": tSNE_2, colname: feature_val}
    return res


def marker_table(marker_df):
    """
    return html code
    """
    marker_df = marker_df.loc[:, ["cluster", "gene",
                                  "avg_logFC", "pct.1", "pct.2", "p_val_adj"]]
    marker_df["cluster"] = marker_df["cluster"].apply(lambda x: f"cluster {x}")
    marker_gene_table = marker_df.to_html(
        escape=False,
        index=False,
        table_id="marker_gene_table",
        justify="center")
    return marker_gene_table


def analysis_smk(args):
    logger1.info('smk analysis ...!')

    # check dir
    outdir = args.outdir
    sample = args.sample
    tsne_tag_file = args.tsne_tag_file
    match_dir = args.match_dir

    if not os.path.exists(outdir):
        os.system('mkdir -p %s' % (outdir))

    # report
    tsne_df_file = glob.glob(f'{match_dir}/*analysis*/*tsne_coord.tsv')[0]
    marker_df_file = glob.glob(f'{match_dir}/*analysis*/*markers.tsv')[0]
    tsne_df = pd.read_csv(tsne_df_file, sep="\t")
    marker_df = pd.read_csv(marker_df_file, sep="\t")
    tsne_tag_df = pd.read_csv(tsne_tag_file, sep="\t", index_col=0)

    report_prepare(outdir, tsne_df, marker_df, tsne_tag_df)

    t = reporter(
        name='analysis_smk',
        assay=args.assay,
        sample=sample,
        outdir=args.outdir +
        '/..')
    t.get_report()


def get_opts_analysis_smk(parser, sub_program):
    if sub_program:
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument('--match_dir', help='match_dir', required=True)
        parser.add_argument(
            '--tsne_tag_file',
            help='smk tsne tag file',
            required=True)
        parser.add_argument('--assay', help='assay', required=True)
